import json
import re
from datetime import datetime, timedelta

import scrapy

from feeds.loaders import FeedEntryItemLoader
from feeds.spiders import FeedsSpider


class DelinskiAtSpider(FeedsSpider):
    name = "delinski.at"

    feed_title = "Delinski"
    feed_link = "https://{}".format(name)
    feed_logo = "https://{}/favicon.ico".format(name)

    def start_requests(self):
        yield scrapy.Request(
            "https://www.delinski.at/wien/restaurants",
            # The restaurants page is not cached and takes a few seconds to load.
            # Don't query more than once a day.
            meta={"cache_expires": timedelta(days=1)},
        )

    def parse(self, response):
        m = re.search("window.DELINSKI, {listViewEntities: (.*)}", response.text)
        restaurants = sorted(
            json.loads(m.group(1))["restaurants"]["entities"].values(),
            key=lambda r: int(r["created"]),
            reverse=True,
        )
        for restaurant in restaurants[:20]:
            il = FeedEntryItemLoader(timezone="UTC", base_url=response.url)
            url = response.urljoin(restaurant["url"])
            il.add_value("link", url)
            il.add_value("title", restaurant["name"])
            content = """
            <img src="{image}">
            <ul>
                <li>{address}</li>
                <li>{price_range_human}</li>
                <li>{cuisine_text}</li>
            </ul>
            """
            il.add_value("content_html", content.format(**restaurant))
            il.add_value(
                "updated", datetime.utcfromtimestamp(int(restaurant["created"]))
            )
            yield scrapy.Request(url, self._parse_restaurant, meta={"il": il})

    def _parse_restaurant(self, response):
        il = FeedEntryItemLoader(
            response=response,
            base_url=response.url,
            parent=response.meta["il"],
            remove_elems=[".external"],
        )
        il.add_css("content_html", ".content .right p")
        il.add_css("content_html", ".restaurant-link")
        il.add_css("category", ".tags a ::text")
        yield il.load_item()



import json
import re

import scrapy
from inline_requests import inline_requests

from feeds.loaders import FeedEntryItemLoader
from feeds.spiders import FeedsSpider


class TuWienAcAtSpider(FeedsSpider):
    name = "tuwien.ac.at"

    feed_title = "TU Wien: Mitteilungsblätter"
    feed_icon = "https://{}/favicon.ico".format(name)

    def start_requests(self):
        yield scrapy.Request(
            "https://tiss.{}/mbl/main/uebersicht".format(self.name),
            headers={"Accept-Language": "de-DE,de"},
            meta={"dont_cache": True},
        )

    @inline_requests
    def parse(self, response):
        mitteilungsblaetter = response.css(".mitteilungsblaetter")
        updated = mitteilungsblaetter.css("::text").re_first(r"(\d{2}\.\d{2}\.\d{4})")
        link = response.urljoin(
            mitteilungsblaetter.css('a::attr("href")').extract_first()
        )

        response = yield scrapy.Request(link, method="HEAD")
        mb_url = response.url
        match = re.search(
            r"https://tiss.tuwien.ac.at/mbl/blatt_struktur/anzeigen/(\d+)", mb_url
        )
        if not match:
            self.logger.error("No Mitteilungsblätter found!")
            return
        else:
            mb_id = match.group(1)

        url = "https://tiss.{}/api/mbl/v22/id/{}".format(self.name, mb_id)
        response = yield scrapy.Request(url)

        last_entry = None
        for entry in reversed(json.loads(response.text)["knoten"]):
            (entry["main"], entry["sub"]) = re.match(
                r"(\d+)\.?(\d*)", entry["counter"]
            ).groups()
            if last_entry is not None and last_entry["main"] == entry["main"]:
                entry["inhalt"] += "<h2>{}</h2>".format(last_entry["titel"])
                entry["inhalt"] += last_entry["inhalt"]
            if entry["sub"] == "":
                il = FeedEntryItemLoader(
                    base_url="https://tiss.{}".format(self.name),
                    timezone="Europe/Vienna",
                    dayfirst=True,
                )
                il.add_value("updated", updated)
                il.add_value("link", mb_url + "#{}".format(entry["counter"]))
                il.add_value("title", entry["titel"])
                il.add_value("content_html", entry["inhalt"])
                yield il.load_item()
                last_entry = None
            else:
                last_entry = entry



import re
from datetime import datetime

from dateutil import rrule
from dateutil.tz import gettz
from scrapy import Request

from feeds.exceptions import DropResponse
from feeds.loaders import FeedEntryItemLoader
from feeds.spiders import FeedsSpider


class ServusTvComSpider(FeedsSpider):
    name = "servustv.com"

    feed_title = "ServusTV"
    feed_link = "https://www.servustv.com"

    def start_requests(self):
        today = datetime.now(gettz("Europe/Vienna")).replace(hour=0, minute=0, second=0)
        for day in rrule.rrule(freq=rrule.DAILY, count=14, dtstart=today):
            yield Request(
                "https://www.servustv.com/wp-admin/admin-ajax.php?"
                + "action=rbmh_tv_program_select_date&date={}".format(
                    day.strftime("%Y-%m-%d")
                ),
                # meta={"dont_cache": True},
            )

    def parse(self, response):
        for url in response.css('.component__card--link::attr("href")').extract():
            yield Request(url, self._parse_video_page)

    def _parse_video_page(self, response):
        match = re.search(
            r"https?://(?:www\.)?servustv\.com/videos/(?P<id>[aA]{2}-\w+|\d+-\d+)",
            response.url,
        )
        if not match:
            return
        video_id = match.group("id").upper()

        il = FeedEntryItemLoader(response=response)
        il.add_value("link", response.url)
        section = response.css(
            "meta[property='article:section']::attr('content')"
        ).extract_first()
        if section != "Allgemein":
            il.add_value("title", section)
        il.add_css("title", "title::text", re="(.*) - Servus TV")
        image_url = response.css(
            "meta[property='og:image']::attr('content')"
        ).extract_first()
        il.add_value("content_html", '<img src="{}">'.format(image_url))
        il.add_css("content_html", "meta[property='og:description']::attr('content')")
        il.add_css("content_html", "#media-asset-content-container")

        match = re.search(r'"dateModified":\s*"([^"]+)"', response.text)
        if match:
            il.add_value("updated", match.group(1))

        stream_url = "https://stv.rbmbtnx.net/api/v1/manifests/%s.m3u8" % video_id

        yield Request(stream_url, self._parse_stream, meta={"il": il})

    def _parse_stream(self, response):
        il = response.meta["il"]

        if response.status != 200:
            url = il.get_output_value("link")
            raise DropResponse(
                "Skipping {} because not downloadable yet".format(url), transient=True
            )

        yield il.load_item()



from urllib.parse import urlparse

import scrapy

from feeds.loaders import FeedEntryItemLoader
from feeds.spiders import FeedsSpider
from feeds.utils import generate_feed_header


class WienerZeitungAtSpider(FeedsSpider):
    name = "wienerzeitung.at"

    _titles = {}
    _ressorts = set()

    def start_requests(self):
        ressorts = self.settings.get("FEEDS_SPIDER_WIENERZEITUNG_AT_RESSORTS")
        if ressorts:
            self._ressorts = set(ressorts.split())
        else:
            self.logger.error("No ressorts given!")
            return

        for ressort in self._ressorts:
            yield scrapy.Request(
                "https://www.{}/{}".format(self.name, ressort),
                meta={"dont_cache": True, "ressort": ressort},
            )

    def feed_headers(self):
        for ressort in self._ressorts:
            yield generate_feed_header(
                title="Wiener Zeitung › {}".format(self._titles.get(ressort, ressort)),
                link="https://www.{}".format(self.name),
                icon="https://www.{}/_em_daten/wzo/favicon.ico".format(self.name),
                logo="https://www.{}/_em_daten/wzo/_layout/logo_rss.png".format(
                    self.name
                ),
                path=ressort,
            )

    def parse(self, response):
        for link in response.css(
            ".topnews-headline::attr(href), .card-title::attr(href)"
        ).extract():
            yield scrapy.Request(
                link + "?em_no_split=1",
                self._parse_article,
                meta={"ressort": response.meta["ressort"]},
            )

    def _parse_article(self, response):
        def _fix_img_src(elem):
            if "data-src-retina" in elem.attrib:
                elem.attrib["src"] = elem.attrib["data-src-retina"]
            elif "data-src" in elem.attrib:
                elem.attrib["src"] = elem.attrib["data-src"]
            return elem

        def _parse_breadcrumbs(breadcrumbs):
            links = breadcrumbs.css("a::text, a::attr('href')").extract()
            # Skip first and last "/" in URL; skip "Startseite" in breadcrumbs.
            return {urlparse(k).path[1:-1]: v for k, v in zip(links[2::2], links[3::2])}

        breadcrumbs = _parse_breadcrumbs(response.css(".breadcrumb a"))
        self._titles = {**self._titles, **breadcrumbs}

        remove_elems = [
            "noscript",
            "h1",
            ".article-meta",
            ".article-header > span.tag",
            ".article-toolbar-head",
            ".figure-copyright",
            ".new-pictures",
            ".author-item",
            ".related-articles",
            'div[data-type="advert"]',
            ".hidden",
            ".article-keywords",
            ".caption-socials",
            ".caption-text > small.d-block",
            "h2 > br",
            "h3 > br",
        ]
        change_tags = {
            ".article-subtitle": "strong",
            "aside": "blockquote",
            "span[style='font-weight: bold;']": "strong",
            "span[style='font-style: italic;']": "em",
            ".container-inner": "blockquote",
        }
        replace_elems = {"img": _fix_img_src}
        il = FeedEntryItemLoader(
            response=response,
            base_url="https://www.{}".format(self.name),
            remove_elems=remove_elems,
            change_tags=change_tags,
            replace_elems=replace_elems,
            timezone="Europe/Vienna",
        )
        il.add_value("link", response.url)
        il.add_css("title", 'meta[property="og:title"]::attr(content)')
        il.add_css("author_name", ".author-headline ::text")
        il.add_value("path", response.meta["ressort"])
        il.add_value("category", breadcrumbs.values())
        il.add_css("category", ".article-keywords li ::text")
        il.add_css("updated", ".article-updated time::attr('datetime')")
        il.add_css("updated", ".article-published time::attr('datetime')")
        il.add_css("content_html", "article")
        return il.load_item()



import scrapy

from feeds.loaders import FeedEntryItemLoader
from feeds.spiders import FeedsSpider


class PythonPatternsGuide(FeedsSpider):
    name = "python-patterns.guide"
    start_urls = ["https://{}".format(name)]

    feed_title = "Python Patterns"
    feed_link = "https://{}".format(name)

    def parse(self, response):
        for path in response.css(".toctree-l1 > a::attr(href)").extract():
            yield scrapy.Request(response.urljoin(path), self._parse_article)

    def _parse_article(self, response):
        remove_elems = ["h1", "#contents", ".headerlink"]
        change_tags = {".admonition-title": "h2"}
        il = FeedEntryItemLoader(
            response=response,
            base_url=response.url,
            remove_elems=remove_elems,
            change_tags=change_tags,
        )
        il.add_value("link", response.url)
        il.add_value("author_name", "Brandon Rhodes")
        # Use "Last-Modified" field or fall back to "Date".
        updated = (
            response.headers.get("Last-Modified", response.headers.get("Date"))
        ).decode("ascii")
        il.add_value("updated", updated)
        il.add_css("title", "title::text")
        il.add_css("content_html", ".section")
        return il.load_item()



import hashlib
import logging
import os
import pickle
import shutil
from collections import defaultdict
from datetime import datetime, timezone
from time import time

import scrapy
from scrapy.extensions.httpcache import DummyPolicy, FilesystemCacheStorage
from scrapy.utils.python import to_bytes
from scrapy.utils.request import request_fingerprint

logger = logging.getLogger(__name__)


class FeedsCachePolicy(DummyPolicy):
    def should_cache_response(self, response, request):
        # We cache all responses regardless of HTTP code.
        return True


class FeedsCache:
    def __init__(self, settings):
        if settings.getbool("HTTPCACHE_ENABLED"):
            self.storage = FeedsCacheStorage(settings)
        else:
            self.storage = FeedsCacheInMemoryStorage()

    def get(self, spider, key):
        return self.storage.retrieve_object(spider, key)

    def set(self, spider, key, obj):
        return self.storage.store_object(spider, key, obj)

    def setdefault(self, spider, key, default_obj):
        obj = self.storage.retrieve_object(spider, key)
        if obj is not None:
            return obj
        self.storage.store_object(spider, key, default_obj)
        return default_obj

    def cleanup(self):
        self.storage.cleanup()


class FeedsCacheStorage(FilesystemCacheStorage):
    def __init__(self, settings):
        super().__init__(settings)
        # gzip is not supported
        self.use_gzip = False
        self._open = open
        self.ignore_http_codes = [
            int(x) for x in settings.getlist("HTTPCACHE_IGNORE_HTTP_CODES")
        ]

    def retrieve_response(self, spider, request):
        """Return response if present in cache, or None otherwise."""
        metadata = self._read_meta(spider, request)
        if metadata is not None and metadata["status"] in self.ignore_http_codes:
            # ignore cache entry for error responses
            logger.debug("Response for {} not cached".format(request))
            return
        # Retrieve response from cache.
        try:
            return super().retrieve_response(spider, request)
        finally:
            logger.debug("Retrieved response for {} from cache".format(request))

    def store_response(self, spider, request, response):
        """Store the given response in the cache."""
        # Read the old metadata.
        old_metadata = self._read_meta(spider, request)
        # This will overwrite old metadata (if there is one).
        super().store_response(spider, request, response)
        # Read the new metadata.
        metadata = self._read_meta(spider, request)
        # Add the parents' fingerprints to the metadata and merge the parents from the
        # old metadata. The last fingerprint is not included since it's the fingerprint
        # of this request.
        metadata["parents"] = list(
            set(request.meta["fingerprints"][:-1]).union(
                old_metadata["parents"] if old_metadata else []
            )
        )
        if request.meta.get("cache_expires") is not None:
            metadata["cache_expires"] = request.meta["cache_expires"].total_seconds()
        metadata["type"] = "response"
        # Write it back.
        rpath = self._get_request_path(spider, request)
        self._write_meta_to_path(rpath, metadata)

    def _get_request_path(self, spider, request):
        key = request_fingerprint(request, include_headers=["Cookie"])
        return os.path.join(self.cachedir, spider.name, key[0:2], key)

    def retrieve_object(self, spider, key):
        metadata = self._read_meta(spider, key)
        if metadata is None:
            return None
        path = self._get_key_path(spider, key)
        with self._open(os.path.join(path, "object"), "rb") as f:
            return pickle.load(f)

    def store_object(self, spider, key, obj):
        path = self._get_key_path(spider, key)
        if not os.path.exists(path):
            os.makedirs(path)
        metadata = {"timestamp": time(), "type": "object"}
        self._write_meta_to_path(path, metadata)
        with self._open(os.path.join(path, "object"), "wb") as f:
            pickle.dump(obj, f, protocol=2)

    def _get_key_path(self, spider, key):
        key = hashlib.sha1(to_bytes(key)).hexdigest()
        return os.path.join(self.cachedir, spider.name, key[0:2], key)

    def remove_response(self, response, spider):
        self.remove_cache_entry(
            self._get_request_path(spider, response.request), remove_parents=True
        )

    def _read_meta(self, spider, key):
        if isinstance(key, scrapy.Request):
            path = self._get_request_path(spider, key)
        else:
            path = self._get_key_path(spider, key)
        return self._read_meta_from_path(path)

    def _read_meta_from_path(self, path):
        try:
            with open(os.path.join(path, "pickled_meta"), "rb") as f:
                return pickle.load(f)
        except FileNotFoundError:
            return None

    def _write_meta_to_path(self, path, metadata):
        with self._open(os.path.join(path, "meta"), "wb") as f:
            f.write(to_bytes(repr(metadata)))
        with self._open(os.path.join(path, "pickled_meta"), "wb") as f:
            pickle.dump(metadata, f, protocol=2)

    def cleanup(self):
        """Removes cache entries in path.

        Entries are removed if one of the conditions is true:
          - Response has a certain status code (e.g. 404).
          - Individual expiration date is reached (compared to now).
          - Timestamp of entry and expires exceeds now.
        """

        logger.debug("Cleaning cache entries from {} ...".format(self.cachedir))

        now = int(datetime.now(timezone.utc).timestamp())
        for cache_entry_path, _dirs, files in os.walk(self.cachedir, topdown=False):
            if "pickled_meta" in files:
                meta = self._read_meta_from_path(cache_entry_path)
                entry_expires_after = min(
                    meta.get("cache_expires", self.expiration_secs),
                    self.expiration_secs,
                )
                threshold = meta["timestamp"] + entry_expires_after
                if now > threshold:
                    self.remove_cache_entry(cache_entry_path)
                elif (
                    meta.get("type", "response") == "response"
                    and meta["status"] in self.ignore_http_codes
                ):
                    self.remove_cache_entry(cache_entry_path, remove_parents=True)
            elif not os.path.samefile(cache_entry_path, self.cachedir):
                # Try to delete parent directory of cache entries.
                try:
                    os.rmdir(cache_entry_path)
                except OSError:
                    # Not empty, don't care.
                    pass

        logger.debug("Finished cleaning cache entries.")

    def remove_cache_entry(self, cache_entry_path, remove_parents=False):
        meta = self._read_meta_from_path(cache_entry_path)
        if meta is None:
            return

        if remove_parents and "parents" in meta:
            spider_root = os.path.dirname(os.path.dirname(cache_entry_path))
            for fingerprint in meta["parents"]:
                path = os.path.join(spider_root, fingerprint[0:2], fingerprint)
                self.remove_cache_entry(path, remove_parents=False)

        shutil.rmtree(cache_entry_path, ignore_errors=True)


class FeedsCacheInMemoryStorage:
    def __init__(self):
        self.data = defaultdict(dict)

    def retrieve_object(self, spider, key):
        return self.data[spider].get(key)

    def store_object(self, spider, key, obj):
        self.data[spider][key] = obj

    def cleanup(self):
        pass



class FeedsException(Exception):
    """
    Base exception class for all exceptions thrown by Feeds.
    """


class DropResponse(FeedsException):
    def __init__(self, message, transient=False):
        self.transient = transient
        super().__init__(message)



import json

import scrapy

from feeds.loaders import FeedEntryItemLoader
from feeds.spiders import FeedsSpider


class Oe1OrfAtSpider(FeedsSpider):
    name = "oe1.orf.at"
    start_urls = ["https://audioapi.orf.at/oe1/api/json/current/broadcasts"]

    feed_title = "oe1.ORF.at"
    feed_subtitle = "Ö1 Webradio"
    feed_link = "https://oe1.orf.at"
    feed_logo = "https://{}/static/img/logo_oe1.png".format(name)

    def parse(self, response):
        for day in json.loads(response.text)[-2:]:
            for broadcast in day["broadcasts"]:
                # Only parse if already recorded (i.e. not live/in the future).
                if broadcast["state"] == "C":
                    yield scrapy.Request(broadcast["href"], self._parse_broadcast)

    def _parse_broadcast(self, response):
        broadcast = json.loads(response.text)
        il = FeedEntryItemLoader(response=response)
        link = "https://{}/player/{}/{}".format(
            self.name, broadcast["broadcastDay"], broadcast["programKey"]
        )
        il.add_value("link", link)
        il.add_value("title", broadcast["programTitle"])
        il.add_value("title", broadcast["title"])
        for stream in broadcast["streams"]:
            stream = "https://loopstream01.apa.at/?channel=oe1&id={}".format(
                stream["loopStreamId"]
            )
            il.add_value("enclosure", {"iri": stream, "type": "audio/mpeg"})
        il.add_value("updated", broadcast["niceTimeISO"])
        if broadcast["subtitle"]:
            il.add_value(
                "content_html", "<strong>{}</strong>".format(broadcast["subtitle"])
            )
        image = broadcast["images"][0]["versions"][0]
        il.add_value(
            "content_html",
            '<img src="{image[path]}" width="{image[width]}">'.format(image=image),
        )
        for item in broadcast["items"]:
            if "title" in item:
                il.add_value("content_html", "<h3>{}</h3>".format(item["title"]))
            il.add_value("content_html", item.get("description"))
        il.add_value("content_html", broadcast["description"])
        il.add_value(
            "content_html",
            '<a href="{broadcast[url]}">{broadcast[urlText]}</a>'.format(
                broadcast=broadcast
            ),
        )
        il.add_value("category", broadcast["tags"])
        if broadcast["url"] and "no_canonical_url" not in broadcast["url"]:
            return scrapy.Request(
                broadcast["url"], self._parse_show, dont_filter=True, meta={"il": il}
            )
        else:
            return il.load_item()

    def _parse_show(self, response):
        il = FeedEntryItemLoader(response=response, parent=response.meta["il"])
        il.add_css("category", ".asideBlock:first-child h2::text")
        return il.load_item()



import json
from datetime import datetime, timedelta

from dateutil.tz import gettz
from scrapy import Request

from feeds.exceptions import DropResponse
from feeds.loaders import FeedEntryItemLoader
from feeds.spiders import FeedsSpider


class TvthekOrfAtSpider(FeedsSpider):
    name = "tvthek.orf.at"
    http_user = "ps_android_v3"
    http_pass = "6a63d4da29c721d4a986fdd31edc9e41"

    feed_title = "TVthek.ORF.at"
    feed_subtitle = "ORF TVTHEK"
    feed_link = "https://tvthek.orf.at"

    def start_requests(self):
        # We only parse today and yesterday because at the end of the day this
        # already produces a lot of requests and feed readers cache previous
        # days (i.e. old contents of our feed) anyways.
        # It's not enough to parse only today because we might miss shows that
        # aired just before midnight but were streamed after midnight
        # (see also https://github.com/nblock/feeds/issues/27)
        today = datetime.now(gettz("Europe/Vienna"))
        for day in [today, today - timedelta(days=1)]:
            yield Request(
                "https://api-tvthek.orf.at/api/v3/schedule/{}?limit=1000".format(
                    day.strftime("%Y-%m-%d")
                ),
                meta={"dont_cache": True},
            )

    def parse(self, response):
        json_response = json.loads(response.text)

        if "next" in json_response["_links"]:
            yield Request(
                json_response["_links"]["nextPage"], meta={"dont_cache": True}
            )

        for item in json_response["_embedded"]["items"]:
            # Skip incomplete items or items with active youth protection.
            # We want to have working download links in the feed item.
            if not item["segments_complete"] or item["has_active_youth_protection"]:
                continue

            # We scrape the episode itself so we can get the segments which are not
            # embedded in the schedule response.
            # Furthermore since this request will be cached, the download URL will also
            # be cached which is convenient for youth protected content.
            yield Request(
                item["_links"]["self"]["href"],
                self._parse_episode,
                # Responses are > 100 KB and useless after 7 days.
                # So don't keep them longer than necessary.
                meta={"cache_expires": timedelta(days=7)},
            )

    def _parse_episode(self, response):
        item = json.loads(response.text)
        il = FeedEntryItemLoader()
        il.add_value("title", item["title"])
        il.add_value(
            "content_html",
            '<img src="{}">'.format(item["playlist"]["preview_image_url"]),
        )
        if item["description"]:
            il.add_value("content_html", item["description"].replace("\r\n", "<br>"))
        il.add_value("updated", item["date"])
        il.add_value("link", item["url"].replace("api-tvthek.orf.at", "tvthek.orf.at"))
        # Check how many segments are part of this episode.
        if len(item["_embedded"]["segments"]) == 1:
            # If only one segment, item["sources"] contains invalid links.
            # We use the first embedded segment instead.
            # This is also how mediathekviewweb.de works.
            item["sources"] = item["_embedded"]["segments"][0]["sources"]
        try:
            video = next(
                s
                for s in item["sources"]["progressive_download"]
                if s["quality_key"] == "Q8C"
            )
            il.add_value("enclosure", {"iri": video["src"], "type": "video/mp4"})
        except StopIteration:
            self.logger.warning(
                "Could not extract video for '{}'!".format(item["title"])
            )
            raise DropResponse(
                "Skipping {} because not downloadable yet".format(response.url),
                transient=True,
            )

        subtitle = item["_embedded"].get("subtitle")
        if subtitle:
            subtitle = subtitle["_embedded"]["srt_file"]["public_urls"]["reference"]
            il.add_value("enclosure", {"iri": subtitle["url"], "type": "text/plain"})
        else:
            self.logger.debug("No subtitle file found for '{}'".format(item["url"]))
        il.add_value(
            "category",
            self._categories_from_oewa_base_path(
                item["_embedded"]["profile"]["oewa_base_path"]
            ),
        )
        return il.load_item()

    def _categories_from_oewa_base_path(self, oewa_base_path):
        """Parse ÖWA Base Path into a list of categories.

         Base paths look like this:

           * RedCont/KulturUndFreizeit/FilmUndKino
           * RedCont/KulturUndFreizeit/Sonstiges
           * RedCont/Lifestyle/EssenUndTrinken
           * RedCont/Nachrichten/Nachrichtenueberblick
           * RedCont/Sport/Sonstiges
        """
        old_new = {
            "RedCont": "",
            "Sonstiges": "",
            "Und": " und ",
            "ue": "ü",
            "ae": "ä",
            "oe": "ö",
            "Ue": "Ü",
            "Ae": "Ä",
            "Oe": "Ö",
        }
        for old, new in old_new.items():
            oewa_base_path = oewa_base_path.replace(old, new)
        return filter(lambda x: x != "", oewa_base_path.split("/"))



import json
from copy import deepcopy
from functools import partial

import lxml
import scrapy
from inline_requests import inline_requests

from feeds.loaders import FeedEntryItemLoader
from feeds.spiders import FeedsXMLFeedSpider
from feeds.utils import generate_feed_header


class AddendumOrgSpider(FeedsXMLFeedSpider):
    name = "addendum.org"
    start_urls = ["https://www.addendum.org/feed/rss2-addendum"]

    _max_articles = 10
    _num_articles = 0

    def feed_headers(self):
        feeds = {"": "Addendum", "podcast": "Addendum Podcast"}
        for path, title in feeds.items():
            yield generate_feed_header(
                title=title,
                path=path,
                subtitle="das, was fehlt",
                link="https://www.{}".format(self.name),
                icon=(
                    "https://www.{}/resources/dist/favicons/android-chrome-192x192.png"
                ).format(self.name),
            )

    def parse_node(self, response, node):
        url = node.xpath("link/text()").extract_first()
        if not node.xpath("category"):
            # Overview pages don't have a category.
            return
        if self._num_articles >= self._max_articles:
            # Maximum number of articles reached.
            return
        self._num_articles += 1
        return scrapy.Request(url, self._parse_article)

    @staticmethod
    def _build_api_request(video_id):
        return scrapy.Request(
            "https://edge.api.brightcove.com/playback/v1/accounts/5548093587001/"
            + "videos/{}".format(video_id),
            headers={
                "Accept": "application/json;pk="
                + "BCpkADawqM0FoC-KqFQHZMCJQrN5XC_gdWIYO4204LOOSFrv34GdVavc7TJP"
                + "tw5F612ztVUZmw47U2kWmyVU3kdtE6dga_P110le86FuaNCGPYlJ6ljW0Z3_"
                + "HYTlnFsPeFs8F61PGWYBsYnG"
            },
        )

    @inline_requests
    def _parse_article(self, response):
        def _inline_video(videos, elem):
            if "data-video-id" in elem.attrib:
                source = lxml.etree.Element("source")
                source.attrib["src"] = videos[elem.attrib["data-video-id"]]
                source.attrib["type"] = "video/mp4"
                elem.insert(0, source)
                return elem
            else:
                # Header video, replace with placeholder image.
                parent = elem.getparent()
                parent.tag = "figure"
                if "data-placeholderbig" in elem.attrib:
                    src = elem.attrib["data-placeholderbig"]
                else:
                    src = elem.attrib["data-placeholder"]
                image = lxml.etree.Element("img")
                image.attrib["src"] = src
                return image

        def _inline_picture(elem):
            elem.tag = "img"
            src = elem.attrib.get("data-original")
            data_min_width = 1000 if src else -1
            for child in elem.getchildren():
                if child.tag != "span":
                    continue
                if int(child.attrib.get("data-min-width", 0)) > data_min_width:
                    src = child.attrib["data-src"]
                    data_min_width = int(child.attrib.get("data-min-width", 0))
                child.drop_tree()
            elem.attrib["src"] = src
            return elem

        audio_ids = response.css(
            '#BCaudioPlayer_eindeutig::attr("data-video-id")'
        ).extract()
        video_ids = response.css('.video-js::attr("data-video-id")').extract()
        media = {}
        for media_id in audio_ids + video_ids:
            api_response = yield self._build_api_request(media_id)
            api_response = json.loads(api_response.text)
            media[media_id] = sorted(
                (
                    video
                    for video in api_response["sources"]
                    if "src" in video and video.get("container") == "MP4"
                ),
                key=lambda v: v["size"],
            )[-1]["src"]

        remove_elems = [
            "h1",
            "script",
            "style",
            ".projectNav",
            ".socialShare",
            ".socialShare__headline",
            ".socialShare__icon",
            ".socialMedia",
            ".socialMedia__headline",
            ".whyRead",
            ".overlayCTA",
            ".authors",
            ".sectionBackground--colorTheme1",
            ".heroStage__copyright",
            ".heroStage__downLink",
            ".callToAction",
            ".print-action",
            ".internalLink span",
            ".addCommunity",
            ".download",
            ".BCaudioPlayer",
            ".icon-date",
            ".callToAction__button",
            'a[href^="http://partners.webmasterplan.com/click.asp"]',
            ".relatedSlider",
            ".imageLightbox",
            ".image__copyrightWrapper",
            ".image__zoom",
            ".image > .picture",
            ".imageHC",
        ]
        change_tags = {
            "div.heroStage__introText": "strong",
            ".quote": "blockquote",
            ".quote__label": "footer",
            ".supernumber": "blockquote",
            ".image": "figure",
            ".image__element": "div",
        }
        replace_elems = {
            "video": partial(_inline_video, media),
            ".picture": _inline_picture,
        }
        pullup_elems = {".image__content figcaption": 3}
        il = FeedEntryItemLoader(
            response=response,
            base_url=response.url,
            remove_elems=remove_elems,
            change_tags=change_tags,
            replace_elems=replace_elems,
            pullup_elems=pullup_elems,
        )
        il.add_value("link", response.url)
        il.add_css("author_name", ".sidebar .authors__name::text")
        il.add_css("title", "title::text", re="(.*) - Addendum")
        il.add_css("updated", 'meta[property="article:modified_time"]::attr(content)')
        # If not yet modified:
        il.add_css("updated", 'meta[property="article:published_time"]::attr(content)')
        il.add_css("content_html", ".content")
        for medium_id, medium_url in media.items():
            if medium_id not in audio_ids:
                il.add_value("enclosure", {"iri": medium_url, "type": "video/mp4"})
        item = il.load_item()
        # Save a copy before yielding it.
        item_podcast = deepcopy(item)
        yield item

        if audio_ids:
            # Export to podcast feed.
            il = FeedEntryItemLoader(item=item_podcast)
            il.add_value("path", "podcast")
            for medium_id, medium_url in media.items():
                if medium_id in audio_ids:
                    il.add_value("enclosure", {"iri": medium_url, "type": "audio/mp4"})
            yield il.load_item()



import logging
import os
from copy import deepcopy
from urllib.parse import urljoin

from lxml import etree
from scrapy.exporters import BaseItemExporter

from feeds.items import FeedEntryItem, FeedItem

logger = logging.getLogger(__name__)


class AtomExporter(BaseItemExporter):
    class AtomFeed(object):
        def __init__(self, exporter, link_self=None):
            self._exporter = exporter
            self._link_self = link_self
            self._feed_updated = None
            self._feed_items = []
            self._xml = etree.Element(
                "feed", nsmap={None: "http://www.w3.org/2005/Atom"}
            )
            self._ids = set()

        def add_item(self, item):
            if isinstance(item, FeedItem):
                if self._link_self:
                    item["link_self"] = self._link_self
                for child in self._convert_feed_item(item):
                    self._xml.insert(0, child)
            elif isinstance(item, FeedEntryItem):
                if item["id"] not in self._ids:
                    self._ids.add(item["id"])
                    entry = etree.Element("entry")
                    for child in self._convert_feed_item(item):
                        entry.append(child)
                    self._feed_items.append(entry)
                else:
                    logger.debug(
                        "Feed entry with id '{}' already in feed.".format(item["id"])
                    )

        def insert_updated(self):
            child = etree.Element("updated")
            child.text = self._feed_updated
            self._xml.insert(0, child)

        def sort(self, fields=("updated", "id"), default=0, reverse=True):
            for item in sorted(
                self._feed_items,
                reverse=reverse,
                key=lambda k: tuple(
                    k.findtext(field, default=default) for field in fields
                ),
            ):
                self._xml.append(item)

        def tostring(self, **kwargs):
            return etree.tostring(self._xml, **kwargs)

        def __len__(self):
            return len(self._feed_items)

        def _convert_feed_item(self, item):
            xml_items = []

            # Convert author related fields.
            author_item = self._convert_special_nested(
                item, "author", ("name", "email")
            )
            if author_item is not None:
                xml_items.append(author_item)

            # Convert link
            key = "link"
            if key in item:
                xml_items.append(self._convert_special_link(item, key, "alternate"))
                item.pop(key)

            # Convert link
            key = "link_self"
            if key in item:
                xml_items.append(self._convert_special_link(item, key, "self"))
                item.pop(key)

            # Convert content
            for key in ("content_text", "content_html"):
                if key in item:
                    xml_items.append(self._convert_special_content(item, key))
                    item.pop(key)

            # Convert enclosure
            key = "enclosure"
            if key in item:
                for enclosure in self._convert_special_enclosure(item, key):
                    xml_items.append(enclosure)
                item.pop(key)

            key = "category"
            if key in item:
                for category in self._convert_special_category(item, key):
                    xml_items.append(category)
                item.pop(key)

            # Convert remaining fields.
            for name, value in self._exporter._get_serialized_fields(
                item, default_value=""
            ):
                element = etree.Element(name)
                element.text = value
                xml_items.append(element)
                if name == "updated":
                    self._update_updated(value)

            return xml_items

        def _convert_special_nested(self, item, parent, children, sep="_"):
            children_items = []
            for full_key in [sep.join((parent, child)) for child in children]:
                if full_key in item:
                    children_items.append(
                        self._convert_special_single_element(item, full_key, sep)
                    )
                    item.pop(full_key)
            if children_items:
                element = etree.Element(parent)
                for children_item in children_items:
                    element.append(children_item)
                return element

        def _convert_special_single_element(self, item, key, sep):
            _, child_key = key.split(sep)
            element = etree.Element(child_key)
            element.text = item[key]
            return element

        def _convert_special_link(self, item, key, rel):
            xml_item = etree.Element("link")
            xml_item.set("rel", rel)
            xml_item.set("href", item[key])
            return xml_item

        def _convert_special_content(self, item, key):
            element_name, content_type = key.split("_")
            xml_item = etree.Element(element_name)
            xml_item.set("type", content_type)
            xml_item.text = item[key]
            return xml_item

        def _convert_special_enclosure(self, item, key):
            for enclosure in item[key]:
                xml_item = etree.Element("link")
                xml_item.set("rel", "enclosure")
                xml_item.set("href", enclosure["iri"])
                xml_item.set("type", enclosure["type"])
                yield xml_item

        def _convert_special_category(self, item, key):
            for category in item[key]:
                xml_item = etree.Element("category")
                xml_item.set("term", category)
                yield xml_item

        def _update_updated(self, raw_updated):
            if raw_updated is None:
                return

            if self._feed_updated is None or self._feed_updated < raw_updated:
                self._feed_updated = raw_updated

    def __init__(self, output_path, output_url, name, **kwargs):
        self._configure(kwargs)
        self._output_path = output_path
        self._output_url = output_url
        self._name = name
        self._feeds = {}
        self._pretty_print = kwargs.pop("pretty_print", True)

    def finish_exporting(self):
        for path, feed in self._feeds.items():
            path = os.path.join(self._output_path, path)
            if len(feed) == 0:
                logger.warning("Feed '{}' contains no items!".format(path))

            feed.insert_updated()
            feed.sort()
            feed = feed.tostring(
                encoding=self.encoding,
                pretty_print=self._pretty_print,
                xml_declaration=True,
            )
            os.makedirs(os.path.dirname(path), exist_ok=True)
            try:
                with open(path, "rb") as f:
                    logger.debug("Found existing feed at '{}'".format(path))
                    old_feed = f.read()
            except FileNotFoundError:
                old_feed = None
            if feed != old_feed:
                with open(path, "wb") as f:
                    f.write(feed)
            else:
                logger.debug(
                    "Feed content not changed, not overwriting feed '{}'".format(path)
                )

    def export_item(self, item):
        for path in item.pop("path", [""]):
            path = os.path.join(self._name, path, "feed.atom")
            if path not in self._feeds:
                if self._output_url:
                    link_self = urljoin(self._output_url, path)
                else:
                    link_self = None
                self._feeds[path] = self.AtomFeed(exporter=self, link_self=link_self)
            # add_item() is destructive, so add a copy.
            self._feeds[path].add_item(deepcopy(item))

        # Pop content fields since we don't want to have them in scrapy's debug
        # output.
        item.pop("content_html", None)
        item.pop("content_text", None)



import scrapy

from feeds.loaders import FeedEntryItemLoader
from feeds.spiders import FeedsSpider


class ZeitdiebinAtSpider(FeedsSpider):
    name = "zeit.diebin.at"
    start_urls = ["https://zeit.diebin.at/upcoming"]

    feed_title = "zeitdiebin"
    feed_subtitle = "irgendwas ist immer ..."
    feed_link = "https://{}".format(name)
    feed_logo = "https://{}/favicon.ico".format(name)

    def parse(self, response):
        for link in response.css("a[href*=events]::attr(href)").re(r"events/\d+"):
            yield scrapy.Request(response.urljoin(link), self.parse_item)

    def parse_item(self, response):
        il = FeedEntryItemLoader(
            response=response,
            base_url="{}/".format(self.feed_link),
            timezone="Europe/Vienna",
            dayfirst=True,
            remove_elems=[".ruler", "h1"],
        )
        il.add_css("title", "h1.event-title::text")
        il.add_value("link", response.url)
        il.add_css("content_html", "div#content.container")
        return il.load_item()



from collections import OrderedDict

import scrapy

from feeds.exceptions import DropResponse
from feeds.loaders import FeedEntryItemLoader
from feeds.spiders import FeedsSpider


class KonsumentAtSpider(FeedsSpider):
    name = "konsument.at"
    start_urls = ["https://www.konsument.at/page/das-aktuelle-heft"]
    custom_settings = {"COOKIES_ENABLED": True}

    feed_title = "KONSUMENT.AT"
    feed_subtitle = "Objektiv, unbestechlich, keine Werbung"

    def parse(self, response):
        user = self.settings.get("FEEDS_SPIDER_KONSUMENT_AT_USERNAME")
        pwd = self.settings.get("FEEDS_SPIDER_KONSUMENT_AT_PASSWORD")
        if user and pwd:
            return scrapy.FormRequest.from_response(
                response,
                formcss="#login form",
                formdata=OrderedDict([("user", user), ("pwd", pwd)]),
                callback=self._after_login,
                meta={"dont_cache": True},
            )
        else:
            # Username, password or section not found in feeds.cfg.
            self.logger.info("Login failed: No username or password given")
            # We can still try to scrape the free articles.
            return self._after_login(response)

    def _after_login(self, response):
        if "login_failed" in response.body_as_unicode():
            self.logger.error("Login failed: Username or password wrong")
        for url in response.xpath(
            '//div[@id="content"]//a[text()!="Bestellen"]/@href'
        ).extract():
            yield scrapy.Request(
                response.urljoin(url), callback=self._parse_article_url
            )

    def _parse_article_url(self, response):
        if not response.css("#content"):
            raise DropResponse(
                "Skipping {} since it is empty".format(response.url), transient=True
            )

        if "Fehler" in response.css("h2 ::text").extract_first():
            raise DropResponse(
                "Skipping {} since it returned an error".format(response.url),
                transient=True,
            )

        remove_elems = ['div[style="padding-top:10px;"]']
        il = FeedEntryItemLoader(
            response=response,
            timezone="Europe/Vienna",
            base_url="https://{}".format(self.name),
            dayfirst=True,
            remove_elems=remove_elems,
        )
        il.add_value("link", response.url)
        il.add_value("author_name", "VKI")
        date = response.css(".issue").re_first(
            r"veröffentlicht:\s*([0-9]{2}\.[0-9]{2}\.[0-9]{4})"
        )
        il.add_value("updated", date)
        url = response.xpath('//a[text()="Druckversion"]/@onclick').re_first(
            r"window\.open\('(.*)'\);"
        )
        il.add_css("title", "h1::text")
        if url:
            return scrapy.Request(
                response.urljoin(url), callback=self._parse_article, meta={"il": il}
            )
        else:
            il.add_value("category", "paywalled")
            il.add_css("content_html", ".primary")
            il.add_css("content_html", 'div[style="padding-top:10px;"] > h3')
            return il.load_item()

    def _parse_article(self, response):
        remove_elems = ["#issue", "h1", "#slogan", "#logo", "#footer"]
        il = FeedEntryItemLoader(
            response=response,
            parent=response.meta["il"],
            base_url="https://{}".format(self.name),
            remove_elems=remove_elems,
        )
        il.add_css("content_html", "#page")
        return il.load_item()



import scrapy

from feeds.exceptions import DropResponse
from feeds.loaders import FeedEntryItemLoader
from feeds.spiders import FeedsXMLFeedSpider
from feeds.utils import generate_feed_header


class EconomistComSpider(FeedsXMLFeedSpider):
    name = "economist.com"
    # Don't send too many requests to not trigger the bot detection.
    custom_settings = {"DOWNLOAD_DELAY": 5.0}

    _titles = {}

    def start_requests(self):
        # See https://www.economist.com/rss for a list of ressorts.
        self._ressorts = self.settings.get("FEEDS_SPIDER_ECONOMIST_COM_RESSORTS", [])
        if self._ressorts:
            self._ressorts = set(self._ressorts.split())
        else:
            self.logger.error("No ressorts given!")
            return

        for ressort in self._ressorts:
            yield scrapy.Request(
                "https://www.{}/{}/rss.xml".format(self.name, ressort),
                meta={"dont_cache": True, "ressort": ressort},
            )

    def parse_node(self, response, node):
        if not self._titles.get(response.meta["ressort"]):
            self._titles[response.meta["ressort"]] = response.xpath(
                "//channel/title/text()"
            ).extract_first()

        url = node.xpath("link/text()").extract_first()
        return scrapy.Request(
            url, self._parse_article, meta={"ressort": response.meta["ressort"]}
        )

    def feed_headers(self):
        for ressort in self._ressorts:
            yield generate_feed_header(
                title="The Economist › {}".format(self._titles.get(ressort, ressort)),
                link="https://www.{}".format(self.name),
                icon="https://www.{}/engassets/ico/favicon.f1ea9088.ico".format(
                    self.name
                ),
                logo=(
                    "https://www.{}/engassets/ico/touch-icon-180x180.f1ea9088.png"
                ).format(self.name),
                path=ressort,
            )

    def _parse_article(self, response):
        title = response.css('meta[property="og:title"]::attr(content)').extract_first()
        if not title:
            raise DropResponse(
                "Skipping {} because ran into bot detection".format(response.url),
                transient=True,
            )

        remove_elems = [
            "meta",
            ".ds-share-list",
            ".advert",
            ".layout-article-links",
            ".ds-chapter-list",
            ".layout-article-meta",
        ]
        change_tags = {
            ".article__lead-image": "figure",
            ".article__description": "h2",
            ".article__footnote": "i",
        }
        il = FeedEntryItemLoader(
            response=response,
            base_url="https://{}".format(self.name),
            remove_elems=remove_elems,
            change_tags=change_tags,
        )
        il.add_value("link", response.url)
        il.add_value("title", title)
        il.add_css("updated", "time.article__dateline-datetime::attr('datetime')")
        il.add_css("content_html", ".article__lead-image")
        il.add_css("content_html", ".article__description")
        il.add_css("content_html", ".layout-article-body")
        il.add_value("path", response.meta["ressort"])
        return il.load_item()



import logging
from copy import copy

from scrapy import Request, signals
from scrapy.exceptions import NotConfigured
from scrapy.spidermiddlewares.httperror import HttpError
from scrapy.utils.misc import load_object
from scrapy.utils.request import request_fingerprint

from feeds.exceptions import DropResponse

logger = logging.getLogger(__name__)


class FeedsHttpErrorMiddleware:
    @classmethod
    def from_crawler(cls, crawler):
        return cls()

    def process_spider_exception(self, response, exception, spider):
        if isinstance(exception, HttpError):
            if response.status >= 500:
                # Transient errors usually caused by overloaded sites, updates, short
                # downtimes, etc.
                lgr = logger.info
            else:
                lgr = logger.warning
            lgr(
                "Ignoring response %(response)r: HTTP status code is not "
                "handled or not allowed",
                {"response": response},
                extra={"spider": spider},
            )
            return []


class FeedsHttpCacheMiddleware:
    def __init__(self, settings):
        if not settings.getbool("HTTPCACHE_ENABLED"):
            raise NotConfigured
        self.storage = load_object(settings["HTTPCACHE_STORAGE"])(settings)

    @classmethod
    def from_crawler(cls, crawler):
        mw = cls(crawler.settings)

        # Note: this hook is a bit of a hack to intercept redirections
        crawler.signals.connect(mw.request_scheduled, signal=signals.request_scheduled)

        return mw

    def process_spider_output(self, response, result, spider):
        def _set_fingerprint(response, r):
            if isinstance(r, Request):
                # Chain fingerprints of response.request and new requests together.
                try:
                    r.meta["fingerprints"] = copy(response.request.meta["fingerprints"])
                except KeyError:
                    r.meta["fingerprints"] = []
            return r

        return (_set_fingerprint(response, r) for r in result or ())

    def request_scheduled(self, request, spider):
        try:
            request.meta["fingerprints"] = copy(request.meta["fingerprints"])
        except KeyError:
            request.meta["fingerprints"] = []
        logger.debug(
            "Parent fingerprints for request {}: {}".format(
                request, request.meta["fingerprints"]
            )
        )
        if not request.meta.get("dont_cache", False):
            fingerprint = request_fingerprint(request, include_headers=["Cookie"])
            request.meta["fingerprints"].append(fingerprint)
        else:
            logger.debug("Skipping fingerprinting uncached request {}".format(request))

    def process_spider_exception(self, response, exception, spider):
        # Note that due to Scrapy bug #220 this is *not* called if DropResponse is
        # raised from a generator.
        # See also https://github.com/scrapy/scrapy/issues/220.
        if isinstance(exception, DropResponse):
            if exception.transient:
                lgr = logger.info
            else:
                lgr = logger.warning
            lgr(exception)
            self.storage.remove_response(response, spider)
            return []



from collections import OrderedDict
from datetime import timedelta

import scrapy
from inline_requests import inline_requests

from feeds.loaders import FeedEntryItemLoader
from feeds.spiders import FeedsXMLFeedSpider
from feeds.utils import generate_feed_header


class NachrichtenAtSpider(FeedsXMLFeedSpider):
    name = "nachrichten.at"
    custom_settings = {"COOKIES_ENABLED": True}

    def start_requests(self):
        self._ressorts = self.settings.get("FEEDS_SPIDER_NACHRICHTEN_AT_RESSORTS")
        if self._ressorts:
            self._ressorts = self._ressorts.split()
        else:
            self.logger.info("No ressorts given, falling back to general ressort!")
            self._ressorts = ["nachrichten"]

        self._username = self.settings.get("FEEDS_SPIDER_NACHRICHTEN_AT_USERNAME")
        self._password = self.settings.get("FEEDS_SPIDER_NACHRICHTEN_AT_PASSWORD")
        if self._username and self._password:
            yield from self._login(None)
        else:
            # We can still try to scrape the free articles.
            self.logger.info("Login failed: No username or password given")

        for ressort in self._ressorts:
            yield scrapy.Request(
                "https://www.{}/storage/rss/rss/{}.xml".format(self.name, ressort),
                meta={"ressort": ressort, "dont_cache": True},
            )

    def feed_headers(self):
        for ressort in self._ressorts:
            yield generate_feed_header(
                title="Oberösterreichische Nachrichten {}".format(ressort.title()),
                path=ressort,
                subtitle="OÖN",
                link="https://www.{}".format(self.name),
                icon="https://static1.{}.at/oonup/images/"
                "apple-touch-icon.png".format(self.name),
                logo="https://www.{}/pics/webapp/"
                "touchicon_180x180.png".format(self.name),
            )

    @inline_requests
    def _login(self, response):
        response = yield scrapy.Request(
            "https://www.{}/login/".format(self.name),
            meta={"cache_expires": timedelta(days=14)},
        )
        response = yield scrapy.FormRequest(
            "https://www.{}/login/".format(self.name),
            formdata=OrderedDict(
                [
                    ("user[control][login]", "true"),
                    ("permanent", "checked"),
                    ("username", self._username),
                    ("password", self._password),
                ]
            ),
            meta={"cache_expires": timedelta(days=14)},
        )
        if response and response.css(".notloggedin"):
            # We tried to login but we failed.
            self.logger.error("Login failed: Username or password wrong")

    def parse_node(self, response, node):
        url = node.xpath("link/text()").extract_first()
        return scrapy.Request(
            url.replace("#ref=rss", ""),
            self._parse_article,
            meta={"handle_httpstatus_list": [410], "ressort": response.meta["ressort"]},
        )

    def _parse_article(self, response):
        def _fix_img_src(elem):
            if "data-src" in elem.attrib:
                elem.attrib["src"] = elem.attrib["data-src"]
            return elem

        if response.status == 410:
            # Articles has been deleted.
            return

        remove_elems = [
            ".artDetail__header__container",
            ".artDetail__extImage__copyright",
            "#readspeaker_button1",
            ".artDetail__userOptions",
            ".container__col--hide",
            ".container__col--mdHide",
            ".artDetailMeineThemen__outer",
            ".artDetailAutor__outer",
            ".artDetailMehrZu",
            "div[style='display: none;']",
            ".artDetail__ooenplusOverlay",
            "#mehrRessort",
            "#morePlus",
            "#widgetFreeEpaper",
        ]
        replace_elems = {"img": _fix_img_src}
        il = FeedEntryItemLoader(
            response=response,
            timezone="Europe/Vienna",
            base_url="https://www.{}".format(self.name),
            remove_elems=remove_elems,
            replace_elems=replace_elems,
            dayfirst=True,
            yearfirst=False,
        )
        if response.css(".mainLogin__linkToggle"):
            il.add_value("category", "paywalled")
        il.add_css("link", 'link[rel="canonical"]::attr(href)')
        il.add_css("title", 'meta[property="og:title"]::attr(content)')
        il.add_css("author_name", ".artDetailAutor__headline::text")
        # Mon, 01 Oct 18 13:42:45 +0200
        il.add_css("updated", 'meta[name="date"]::attr(content)')
        il.add_css("content_html", "article.artDetail")
        il.add_css("category", ".artDetailOrt__linkText::text")
        il.add_value("path", response.meta["ressort"])
        return il.load_item()



import json
import re
from collections import OrderedDict
from datetime import timedelta

import scrapy
from dateutil.parser import parse as dateutil_parse
from scrapy.loader.processors import MapCompose

from feeds.loaders import FeedEntryItemLoader
from feeds.spiders import FeedsSpider
from feeds.utils import generate_feed_header


class FalterAtSpider(FeedsSpider):
    name = "falter.at"
    # Don't overwhelm the poor Wordpress with too many requests at once.
    custom_settings = {"DOWNLOAD_DELAY": 1.0, "COOKIES_ENABLED": True}

    def start_requests(self):
        pages = self.settings.get("FEEDS_SPIDER_FALTER_AT_PAGES")
        if pages:
            self.pages = pages.split()
        else:
            self.pages = [
                "lokalfuehrer_reviews",
                "lokalfuehrer_newest",
                "magazine",
                "streams",
            ]

        if "magazine" in self.pages:
            abonr = self.settings.get("FEEDS_SPIDER_FALTER_AT_ABONR")
            password = self.settings.get("FEEDS_SPIDER_FALTER_AT_PASSWORD")
            if abonr and password:
                yield scrapy.FormRequest(
                    url="https://www.{}/login".format(self.name),
                    formdata=OrderedDict(
                        [
                            ("login[abonr]", abonr),
                            ("login[password]", password),
                            ("redirect_url", "/archiv/"),
                        ]
                    ),
                    meta={
                        "dont_redirect": True,
                        "cache_expires": timedelta(hours=3),
                        "handle_httpstatus_list": [302],
                    },
                    callback=self.request_archive,
                )
            else:
                # Username, password or section falter.at not found in feeds.cfg.
                self.logger.info(
                    "Login failed: No username or password given. "
                    + "Only free articles are available in full text."
                )
                yield self.request_archive()

        if "lokalfuehrer_reviews" in self.pages:
            yield scrapy.Request(
                (
                    "https://www.{}/api/lokalfuehrer/search?v=true&"
                    + "sort_pos=front&sort=review.post_date:desc&c=10"
                ).format(self.name),
                self.parse_lokalfuehrer,
                meta={"dont_cache": True, "lokalfuehrer": "reviews"},
            )

        if "lokalfuehrer_newest" in self.pages:
            yield scrapy.Request(
                (
                    "https://www.{}/api/lokalfuehrer/search?"
                    + "sort_pos=front&sort=id:desc&c=20"
                ).format(self.name),
                self.parse_lokalfuehrer,
                meta={"dont_cache": True, "lokalfuehrer": "newest"},
            )

        if "streams" in self.pages:
            yield scrapy.Request(
                ("https://www.{}/api/kino?has_stream=true&mode=movie&c=100").format(
                    self.name
                ),
                self.parse_movies,
                meta={"dont_cache": True, "movies": "streams"},
            )

        blogs = self.settings.get("FEEDS_SPIDER_FALTER_AT_BLOGS")
        if blogs:
            self.blogs = blogs.split()
        else:
            self.blogs = []

        for blog in self.blogs:
            yield scrapy.Request(
                "https://cms.falter.at/blogs/author/" + blog + "/",
                self.parse_blog_overview,
                meta={"dont_cache": True, "blog": blog},
            )

    def request_archive(self, response=None):
        if response and response.status != 302:
            self.logger.error("Login failed: Username or password wrong!")
        else:
            return scrapy.Request(
                "https://www.{}/archiv/".format(self.name),
                self.parse_archive,
                meta={"dont_cache": True},
            )

    def feed_headers(self):
        for path in self.pages:
            yield generate_feed_header(
                title="falter.at",
                subtitle="Wir holen dich da raus.",
                link="https://www.falter.at",
                path=path,
            )

    def parse_lokalfuehrer(self, response):
        entries = json.loads(response.text)["hits"]
        for entry in entries:
            il = FeedEntryItemLoader(
                response=response, base_url="https://{}".format(self.name)
            )
            il.add_value(
                "path", "lokalfuehrer_{}".format(response.meta["lokalfuehrer"])
            )
            il.add_value(
                "link", "https://www.{}/lokal/{}".format(self.name, entry["id"])
            )
            il.add_value("category", entry["categories"])
            il.add_value("category", entry["zip"])
            il.add_value("category", entry["city"])
            review = entry.get("review")
            if review:
                il.add_value("title", review["post_title"])
                il.add_value("title", review["post_subtitle"])
                il.add_value("author_name", review["meta"].split("|")[0].title())
                il.add_value("category", "review")
                il.add_value("updated", review["post_date"])
            else:
                il.add_value("title", entry["name"])
            for picture in entry["pictures"] or []:
                il.add_value(
                    "content_html",
                    '<img src="https://faltercdn2.falter.at/wwei/1080/{}">'.format(
                        picture["filename"]
                    ),
                )
            if review:
                il.add_value("content_html", review["post_content"])
            il.add_value("content_html", entry["category_text"])
            il.add_value(
                "content_html",
                "<p>{} {}, {}</p>".format(entry["zip"], entry["city"], entry["street"]),
            )
            if entry["location"]:
                il.add_value(
                    "content_html",
                    (
                        '<p><a href="https://www.google.com/maps?q={lat},{lon}">'
                        + "Google Maps</a></p>"
                    ).format(**entry["location"]),
                )
            yield il.load_item()

    def parse_movies(self, response):
        entries = json.loads(response.text)["hits"]
        for entry in entries:
            il = FeedEntryItemLoader(
                response=response, base_url="https://{}".format(self.name)
            )
            il.add_value("path", "{}".format(response.meta["movies"]))
            il.add_value(
                "link", "https://www.{}/kino/{}".format(self.name, entry["prod_id"])
            )
            il.add_value("title", entry["prod"])
            il.add_value("content_html", entry["comment"])
            for image in entry["images"] or []:
                il.add_value(
                    "content_html",
                    '<img src="https://faltercdn2.falter.at/events/1080/{}">'.format(
                        image["filename"]
                    ),
                )
            if "stream" in entry:
                il.add_value(
                    "content_html", '<a href="{s}">{s}</a>'.format(s=entry["stream"])
                )
            for key, value in entry.items():
                if key.startswith("has_") and value:
                    il.add_value("category", key.replace("has_", ""))
                elif key.startswith("is_") and value:
                    il.add_value("category", key.replace("is_", ""))
            il.add_value("updated", entry["index_date"])
            yield il.load_item()

    def parse_archive(self, response):
        # The perks of having a JavaScript frontend ...
        issues = json.loads(
            response.css("router-view").re_first("<router-view :data='([^']+)'>")
        )["issues"]
        latest_issue_date = dateutil_parse(
            issues[sorted(issues.keys())[-1]][-1], ignoretz=True
        )
        # The JS frontend calculates the issue number the same way, so this should be
        # somewhat official.
        issuenr = "{0[0]}{0[1]}".format(latest_issue_date.date().isocalendar())
        return scrapy.Request(
            response.urljoin("/api/archive/{}?count=1000&from=0".format(issuenr)),
            self.parse_archive_search,
            meta={"issue_date": latest_issue_date},
        )

    def parse_archive_search(self, response):
        articles = json.loads(response.text)["articles"]["hits"]
        for i, item in enumerate(articles):
            il = FeedEntryItemLoader(
                response=response,
                base_url="https://{}".format(self.name),
                timezone="Europe/Vienna",
            )
            il.add_value("path", "magazine")
            link = response.urljoin(item["detail_link"])
            il.add_value("link", link)
            try:
                author = re.sub(
                    r"(?:.*:|Von)\s*(.*)", r"\1", ", ".join(item["authors"]).title()
                )
                il.add_value("author_name", author)
            except IndexError:
                pass
            il.add_value("title", item["title"])
            # All articles have the same date.
            # We add an offset so they are sorted in the right order.
            date = response.meta["issue_date"] + timedelta(seconds=i)
            il.add_value("updated", date)
            il.add_value("category", item["ressort"])
            yield scrapy.Request(link, self.parse_item_text, meta={"il": il})

    def parse_item_text(self, response):
        remove_elems = [".ad-component", ".wp-caption-text"]
        il = FeedEntryItemLoader(
            response=response,
            parent=response.meta["il"],
            remove_elems=remove_elems,
            base_url="https://{}".format(self.name),
        )
        if response.css(".bluebox"):
            il.add_value("category", "paywalled")
        il.add_css("content_html", "div.pR")
        return il.load_item()

    def parse_blog_overview(self, response):
        yield generate_feed_header(
            title=response.css("article > h1 ::text").extract_first(),
            link="https://www.falter.at",
            path="blog_{}".format(response.meta["blog"]),
        )

        for link in response.css("div[id^=post-] a::attr(href)").extract():
            yield scrapy.Request(
                link, self.parse_blog_article, meta={"blog": response.meta["blog"]}
            )

    def parse_blog_article(self, response):
        remove_elems = [".ad-component", ".wp-caption-text"]
        il = FeedEntryItemLoader(
            response=response,
            remove_elems=remove_elems,
            base_url="https://cms.{}".format(self.name),
            timezone="Europe/Vienna",
            dayfirst=True,
            yearfirst=False,
        )
        il.add_css("content_html", "article > h2")
        il.add_css("content_html", ".storycontent-article")
        il.add_css("author_name", ".falter-heading ::text", MapCompose(str.title))
        il.add_css(
            "author_name", ".thinktank-meta > span ::text", MapCompose(str.title)
        )
        il.add_css("updated", ".post > .text-label ::text", re=r"(\d{2}\.\d{2}\.\d{4})")
        il.add_value("link", response.url)
        il.add_value("path", "blog_{}".format(response.meta["blog"]))
        il.add_css("title", "article > h1 ::text")
        return il.load_item()



import scrapy
from scrapy.http import HtmlResponse

from feeds.loaders import FeedEntryItemLoader
from feeds.spiders import FeedsSpider


class WienerLinienAtSpider(FeedsSpider):
    name = "wienerlinien.at"
    custom_settings = {
        "DEFAULT_REQUEST_HEADERS": {
            "Accept": "text/html",
            # Don't include Accept-Language so the datetime in the HTML
            # response includes the time.
        }
    }
    start_urls = [
        "https://www.wienerlinien.at/eportal3/ep/scrollingListView.do?"
        "scrolling=true&startIndex=0&channelId=-47186&programId=74577"
    ]

    feed_title = "Wiener Linien"
    feed_subtitle = "Aktuelle Meldungen"

    def parse(self, response):
        # Wiener Linien returns HTML with an XML content type which creates an
        # XmlResponse.
        response = HtmlResponse(url=response.url, body=response.body)
        for item in response.css(".block-news-item"):
            il = FeedEntryItemLoader(
                response=response,
                timezone="Europe/Vienna",
                ignoretz=True,
                base_url="https://www.{}".format(self.name),
            )
            link = response.urljoin(item.css("a::attr(href)").extract_first())
            il.add_value("link", link)
            il.add_value("title", item.css("h3::text").extract_first())
            il.add_value("updated", item.css(".date::text").extract_first())
            yield scrapy.Request(link, self.parse_item, meta={"il": il})

    def parse_item(self, response):
        remove_elems = ["h1", ".delayed-image-load"]
        change_tags = {"noscript": "div"}
        il = FeedEntryItemLoader(
            response=response,
            parent=response.meta["il"],
            remove_elems=remove_elems,
            change_tags=change_tags,
            base_url="https://www.{}".format(self.name),
        )
        il.add_xpath("content_html", '//div[@id="main-inner"]')
        return il.load_item()



import json
from collections import OrderedDict
from datetime import timedelta
from urllib.parse import parse_qs, urlparse

import scrapy
from inline_requests import inline_requests
from scrapy.http import FormRequest

from feeds.loaders import FeedEntryItemLoader
from feeds.spiders import FeedsXMLFeedSpider


class UebermedienDeSpider(FeedsXMLFeedSpider):
    name = "uebermedien.de"
    start_urls = ["https://uebermedien.de/feed/"]
    namespaces = [("dc", "http://purl.org/dc/elements/1.1/")]
    custom_settings = {"COOKIES_ENABLED": True}

    feed_title = "uebermedien.de"
    feed_subtitle = "Medien besser kritisieren."
    _steady_token = None

    def start_requests(self):
        self._username = self.settings.get("FEEDS_SPIDER_UEBERMEDIEN_DE_USERNAME")
        self._password = self.settings.get("FEEDS_SPIDER_UEBERMEDIEN_DE_PASSWORD")
        if self._username and self._password:
            yield from self._steady_login(None)
        else:
            # We can still try to scrape the free articles.
            self.logger.info("Login failed: No username or password given")

        yield from super().start_requests()

    @inline_requests
    def _steady_login(self, response):
        response = yield scrapy.Request(
            "https://steadyhq.com/oauth/authorize?"
            + "client_id=0c29f006-1a98-48f1-8a63-2c0652c59f28&"
            + "redirect_uri=https://uebermedien.de&scope=read&"
            + "response_type=code&refresh_only=false",
            meta={"cache_expires": timedelta(days=1)},
        )

        response = yield FormRequest.from_response(
            response,
            formdata=OrderedDict(
                [("user[email]", self._username), ("user[password]", self._password)]
            ),
            dont_filter=True,
            meta={"handle_httpstatus_list": [301], "cache_expires": timedelta(days=1)},
        )

        try:
            code = parse_qs(urlparse(response.url).query)["code"][0]
        except KeyError:
            self.logger.error("Login failed: Wrong username and password")
            return

        body = OrderedDict(
            [
                ("client_id", "0c29f006-1a98-48f1-8a63-2c0652c59f28"),
                ("grant_type", "authorization_code"),
                ("code", code),
                ("redirect_uri", "https://uebermedien.de"),
            ]
        )
        response = yield scrapy.Request(
            "https://steadyhq.com/api/v1/oauth/token",
            method="POST",
            body=json.dumps(body),
            headers={"Accept": "application/json", "Content-Type": "application/json"},
            meta={"cache_expires": timedelta(days=1)},
        )
        self._steady_token = json.loads(response.text)["access_token"]

    def parse_node(self, response, node):
        il = FeedEntryItemLoader(
            response=response, base_url="https://{}".format(self.name), dayfirst=True
        )
        il.add_value("updated", node.xpath("//pubDate/text()").extract_first())
        il.add_value("author_name", node.xpath("//dc:creator/text()").extract_first())
        il.add_value("category", node.xpath("//category/text()").extract())
        title = node.xpath("(//title)[2]/text()").extract()
        if not title:
            # Fallback to the first category if no title is provided (e.g. comic).
            title = node.xpath("//category/text()").extract_first()
        il.add_value("title", title)
        link = node.xpath("(//link)[2]/text()").extract_first()
        il.add_value("link", link)
        if self._steady_token:
            cookies = {"steady-token": self._steady_token}
        else:
            cookies = None
        return scrapy.Request(
            link, self._parse_article, cookies=cookies, meta={"il": il}
        )

    def _parse_article(self, response):
        remove_elems = ["script"]
        convert_footnotes = [".footnoteContent"]
        pullup_elems = {".footnoteContent": 1}
        change_tags = {".entry-content-info-box": "blockquote"}
        il = FeedEntryItemLoader(
            response=response,
            parent=response.meta["il"],
            remove_elems=remove_elems,
            change_tags=change_tags,
            base_url="https://{}".format(self.name),
            convert_footnotes=convert_footnotes,
            pullup_elems=pullup_elems,
        )
        il.add_css("content_html", ".entry-content")
        return il.load_item()



import json
from datetime import datetime, timedelta, timezone

import scrapy

from feeds.loaders import FeedEntryItemLoader
from feeds.spiders import FeedsSpider


class AtvAtSpider(FeedsSpider):
    name = "atv.at"
    start_urls = ["https://atv.at/mediathek/neue-folgen/"]
    custom_settings = {
        # The site is really shitty, don't overwhelm it with more requests.
        "CONCURRENT_REQUESTS": 1
    }

    feed_title = "ATV.at"
    feed_subtitle = "Mediathek"
    _timerange = timedelta(days=7)

    def parse(self, response):
        for link in response.css(".program_link::attr(href)").extract():
            yield scrapy.Request(link, self.parse_item, meta={"dont_cache": True})

        for url in response.css(".topteaser_wrapper::attr(href)").extract():
            yield scrapy.Request(url, self.parse_program)

    def parse_item(self, response):
        for url in response.css(".video").xpath("../../@href").extract():
            yield scrapy.Request(url, self.parse_program)

    def parse_program(self, response):
        if not response.css(r".jsb_video\/FlashPlayer"):
            return
        data = json.loads(
            response.css(r".jsb_video\/FlashPlayer").xpath("@data-jsb").extract()[0]
        )
        data = data["config"]["initial_video"]["parts"][0]["tracking"]["nurago"]
        il = FeedEntryItemLoader(
            response=response,
            base_url="https://{}".format(self.name),
            timezone="Europe/Vienna",
            dayfirst=True,
        )
        il.add_value("link", data["clipurl"])
        il.add_value("title", data["programname"])
        il.add_value("updated", data["airdate"])
        il.add_xpath("content_html", '//p[@class="plot_summary"]')
        item = il.load_item()
        # Only include videos posted in the last 7 days.
        if item["updated"] + self._timerange > datetime.now(timezone.utc):
            return item



import json
from urllib.parse import urljoin

import scrapy

from feeds.loaders import FeedEntryItemLoader
from feeds.spiders import FeedsSpider
from feeds.utils import generate_feed_header


class KurierAtSpider(FeedsSpider):
    name = "kurier.at"

    def feed_headers(self):
        for medium in self._media:
            yield generate_feed_header(
                title="Kurier.at",
                subtitle="Minutenaktuelle Nachrichten aus Österreich und der Welt. "
                + "kurier.at - die österreichische Nachrichten-Plattform im Internet. "
                + "24 hour news from Austria's biggest quality newspaper.",
                path=medium,
                link="https://www.{}".format(self.name),
                logo="https://{}/assets/logos/logo.png".format(self.name),
            )

    def start_requests(self):
        channels = self.settings.get("FEEDS_SPIDER_KURIER_AT_CHANNELS", "").split()
        articles = self.settings.get("FEEDS_SPIDER_KURIER_AT_ARTICLES", "").split()
        authors = self.settings.get("FEEDS_SPIDER_KURIER_AT_AUTHORS", "").split()
        if not any([channels, articles, authors]):
            self.logger.error(
                "Please specify what to crawl for kurier.at in config file!"
            )
            self._media = []
            return

        self._media = channels + articles + authors

        for channel in channels:
            yield scrapy.Request(
                "https://efs.kurier.at/api/v1/cfs/route?uri=/kurierat{}".format(
                    channel
                ),
                self._parse_channel,
                # The response should be stable since we only want to get the collection
                # for the channel so we allow caching.
                meta={"path": channel, "feed_type": "channel"},
            )

        for article in articles:
            yield scrapy.Request(
                "https://efs.kurier.at/api/v1/cfs/route?uri=/kurierat{}".format(
                    article
                ),
                self._parse_article,
                meta={"path": article, "dont_cache": True, "feed_type": "article"},
            )

        for author in authors:
            yield scrapy.Request(
                "https://efs.kurier.at/api/v1/cfs/route?uri=/kurierat"
                + "/author/{}".format(author),
                self._parse_author,
                # The response should be stable since we only want to get the ID for the
                # author so we allow caching.
                meta={"path": author, "feed_type": "author"},
            )

    def _parse_channel(self, response):
        for block in json.loads(response.text)["layout"]["center"]:
            if block["type"] == "longList":
                return scrapy.Request(
                    (
                        "https://efs.kurier.at/api/v1/cfs/collection/{}"
                        + "?start=0&limit=20"
                    ).format(block["collectionName"]),
                    self._parse_collection,
                    meta={
                        "path": response.meta["path"],
                        "dont_cache": True,
                        "feed_type": response.meta["feed_type"],
                    },
                )

    def _parse_collection(self, response):
        articles = json.loads(response.text)["items"]
        for article in articles:
            yield scrapy.Request(
                "https://efs.kurier.at/api/v1/cfs/route?uri=/{}{}".format(
                    article["portal"].replace(".", ""), article["url"]
                ),
                self._parse_article,
                meta={
                    "path": response.meta["path"],
                    "feed_type": response.meta["feed_type"],
                },
            )

    def _create_figure(self, src, caption=None):
        src = urljoin("https://image.{}".format(self.name), src)
        return (
            '<figure><div><img src="{src}"></div>'
            + "<figcaption>{caption}</figcaption></figure>"
        ).format(src=src, caption=caption or "")

    def _parse_article(self, response):
        article = json.loads(response.text)["layout"]["center"][0]
        il = FeedEntryItemLoader()
        il.add_value(
            "link", urljoin("https://{}".format(article["portal"]), article["url"])
        )
        il.add_value("title", article["title"])
        if "teaser_img" in article:
            il.add_value(
                "content_html",
                self._create_figure(
                    article["teaser_img"]["url"],
                    article["teaser_img"].get("description"),
                ),
            )
        il.add_value(
            "content_html", "<p><strong>{}</strong></p>".format(article["teaser_text"])
        )
        for paragraph in article["paragraphs"]:
            if paragraph["type"] == "text":
                il.add_value("content_html", paragraph["data"]["html"])
            elif paragraph["type"] == "youtube":
                url = "https://www.youtube.com/watch?v={}".format(
                    paragraph["data"]["videoid"]
                )
                il.add_value(
                    "content_html",
                    '<div><a href="{url}">{url}</a></div>'.format(url=url),
                )
            elif paragraph["type"] == "image":
                il.add_value(
                    "content_html",
                    self._create_figure(
                        paragraph["data"]["url"].replace("large", "original"),
                        paragraph["data"].get("description"),
                    ),
                )
            elif paragraph["type"] == "gallery":
                # Only include 1 image (the latest) if the feed type is article.
                # This is is a special case for comic articles where a new image is
                # added to the article once a day and it doesn't make sense to always
                # include all the old ones in the feed.
                max_images = 1 if response.meta["feed_type"] == "article" else None
                for image in paragraph["data"]["images"][:max_images]:
                    il.add_value(
                        "content_html",
                        self._create_figure(
                            image["url"].replace("large", "original"),
                            image.get("description"),
                        ),
                    )
        il.add_value("updated", article["updated_date"])
        for author in article["authors"]:
            il.add_value("author_name", "{firstname} {lastname}".format(**author))
        if not article["authors"]:
            il.add_value("author_name", article["agency"])
        il.add_value("category", article["channel"]["name"])
        il.add_value("category", article["portal"])
        il.add_value("path", response.meta["path"])
        if article["sponsored"]:
            il.add_value("category", "sponsored")
        il.add_value("category", article.get("pretitle"))
        return il.load_item()

    def _parse_author(self, response):
        query = json.loads(response.text)["layout"]["center"][0]["query"]
        return scrapy.Request(
            "https://efs.kurier.at/api/v1/cfs/search?query={}&limit=10&page=1".format(
                query
            ),
            self._parse_search,
            meta={
                "path": response.meta["path"],
                "dont_cache": True,
                "feed_type": response.meta["feed_type"],
            },
        )

    def _parse_search(self, response):
        articles = json.loads(response.text)["articles"]
        for article in articles:
            yield scrapy.Request(
                "https://efs.kurier.at/api/v1/cfs/route?uri=/{}{}".format(
                    article["portal"].replace(".", ""), article["url"]
                ),
                self._parse_article,
                meta={
                    "path": response.meta["path"],
                    "feed_type": response.meta["feed_type"],
                },
            )



from urllib.parse import urljoin

import scrapy

from feeds.loaders import FeedEntryItemLoader
from feeds.spiders import FeedsSpider
from feeds.utils import generate_feed_header


class UbupComSpider(FeedsSpider):
    name = "ubup.com"

    _base_url = "https://www.{}".format(name)
    _scrape_pages = 3

    def start_requests(self):
        links = self.settings.get("FEEDS_SPIDER_UBUP_COM_LINKS")
        if links:
            links = links.split()
        else:
            links = ["katalog?sortiertnach=neueste"]

        for link in links:
            yield scrapy.Request(
                urljoin(self._base_url, link), meta={"dont_cache": True, "path": link}
            )

    def feed_headers(self):
        return []

    def parse(self, response):
        if len(response.css(".thumbnail")) == 0:
            self.logger.info("No items found.")
            return

        for item in response.css(".thumbnail"):
            il = FeedEntryItemLoader(selector=item, base_url=self._base_url)
            il.add_css("title", ".item_brand_text ::text")
            il.add_css("title", ".item-title ::text")
            il.add_css("title", ".current-price ::text")
            il.add_value(
                "link",
                response.urljoin(item.css(".item-link::attr(href)").extract_first()),
            )
            image_url = item.css(".item-image::attr(data-bg)").re_first(
                r"url\(([^)]+)\)"
            )
            # Fix broken images.
            if image_url.startswith("https://markenankauf.momox.de/pics/https://"):
                image_url = image_url.replace(
                    "https://markenankauf.momox.de/pics/https://", "https://"
                )
            il.add_value("content_html", '<img src="{}">'.format(image_url))
            il.add_css("content_html", ".item-des-container")
            il.add_value("path", response.meta["path"])
            yield il.load_item()

        page = int(response.css(".pagination .active a::text").extract_first())
        if page == 1:
            yield generate_feed_header(
                title=response.css("title ::text").re_first(
                    "(ubup | .*) Second Hand kaufen"
                ),
                subtitle="Deutschlands größter Second Hand-Onlineshop für "
                "Mode & Accessoires",
                icon="https://www.{}/images/favicon.ico".format(self.name),
                link=response.url,
                path=response.meta["path"],
            )
        if page < self._scrape_pages:
            next_page = response.css(
                ".pagination .active + li a::attr(href)"
            ).extract_first()
            if next_page:
                yield scrapy.Request(
                    response.urljoin(next_page),
                    meta={"dont_cache": True, "path": response.meta["path"]},
                )



from datetime import datetime, timedelta

import scrapy

from feeds.loaders import FeedEntryItemLoader
from feeds.spiders import FeedsSpider
from feeds.utils import generate_feed_header


class DerStandardAtSpider(FeedsSpider):
    name = "derstandard.at"

    _users = {}
    _titles = {}

    def start_requests(self):
        self._ressorts = self.settings.get("FEEDS_SPIDER_DERSTANDARD_AT_RESSORTS", [])
        if self._ressorts:
            self._ressorts = set(self._ressorts.split())
        else:
            self.logger.error("No ressorts given!")
            return

        for ressort in self._ressorts:
            yield scrapy.Request(
                "https://www.{}/{}".format(self.name, ressort),
                meta={"dont_cache": True, "ressort": ressort},
                # Cookie handling is disabled, so we have to send this as a header.
                headers={"Cookie": "DSGVO_ZUSAGE_V1=true"},
            )

        self._users = {
            user_id: None
            for user_id in self.settings.get(
                "FEEDS_SPIDER_DERSTANDARD_AT_USERS", ""
            ).split()
        }
        for user_id in self._users.keys():
            for page in range(3):
                yield scrapy.Request(
                    (
                        "https://apps.{}/userprofil/postings/{}?"
                        + "pageNumber={}&sortMode=1"
                    ).format(self.name, user_id, page),
                    self._parse_user_profile,
                    meta={
                        # Older pages should be cached longer.
                        "cache_expires": timedelta(hours=page),
                        "path": "userprofil/postings/{}".format(user_id),
                        "user_id": user_id,
                    },
                    headers={"Cookie": "DSGVO_ZUSAGE_V1=true"},
                )

    def feed_headers(self):
        for ressort in self._ressorts:
            yield generate_feed_header(
                title="derStandard.at › {}".format(self._titles.get(ressort, ressort)),
                subtitle="Nachrichten in Echtzeit",
                link="https://www.{}".format(self.name),
                icon="https://at.staticfiles.at/sites/mainweb/img/icons/dst/"
                "dst-16.ico",
                logo="https://at.staticfiles.at/sites/mainweb/img/icons/dst/"
                "dst-228.png",
                path=ressort,
            )

        for user_id, name in self._users.items():
            yield generate_feed_header(
                title="derStandard.at › Postings von {}".format(name),
                subtitle="Nachrichten in Echtzeit",
                link="https://apps.{}/userprofil/postings/{}".format(
                    self.name, user_id
                ),
                icon="https://at.staticfiles.at/sites/mainweb/img/icons/dst/dst-16.ico",
                logo="https://at.staticfiles.at/sites/mainweb/img/icons/dst/"
                "dst-228.png",
                path="userprofil/postings/{}".format(user_id),
            )

    def parse(self, response):
        for link in response.css("section[data-type='date'] a::attr('href')").extract():
            yield scrapy.Request(
                response.urljoin(link),
                self._parse_article,
                meta={"ressort": response.meta["ressort"]},
                headers={"Cookie": "DSGVO_ZUSAGE_V1=true"},
            )

    def _parse_article(self, response):
        def _fix_img_src(elem):
            if "src" not in elem.attrib:
                if "data-lazy-src" in elem.attrib:
                    elem.attrib["src"] = elem.attrib["data-lazy-src"]
                elif "data-src" in elem.attrib:
                    elem.attrib["src"] = elem.attrib["data-src"]
            return elem

        def _parse_breadcrumbs(breadcrumbs):
            links = breadcrumbs.css("a::text, a::attr('href')").extract()
            return {k[1:]: v for k, v in zip(links[::2], links[1::2])}

        breadcrumbs = _parse_breadcrumbs(
            response.css(".site-contextnavigation-breadcrumbs-nav a")
        )
        self._titles = {**self._titles, **breadcrumbs}

        remove_elems = [
            "ad-container",
            "figure > footer",
            "picture > button",
            "div[data-section-type='newsletter']",
            ".gallery-summary",
        ]
        change_tags = {
            ".article-subtitle": "strong",
            "aside": "blockquote",
            "p strong:only-child": "h3",
        }
        replace_elems = {"img": _fix_img_src}
        il = FeedEntryItemLoader(
            response=response,
            base_url="https://{}".format(self.name),
            remove_elems=remove_elems,
            change_tags=change_tags,
            replace_elems=replace_elems,
            timezone="Europe/Vienna",
        )
        il.add_value("link", response.url)
        il.add_css("title", 'meta[property="og:title"]::attr(content)')
        if response.css(".article-origins .article-author-avatar"):
            # Blog posts.
            il.add_css("author_name", ".article-author-avatar > span ::text")
        else:
            # Normal articles.
            il.add_css("author_name", ".article-origins ::text")
        il.add_value("path", response.meta["ressort"])
        il.add_value("category", breadcrumbs.values())
        il.add_css("category", ".storylabels span ::text")
        il.add_css("updated", "time::attr('datetime')")
        il.add_css("content_html", ".article-subtitle")
        il.add_css("content_html", ".article-body")
        return il.load_item()

    def _parse_user_profile(self, response):
        self._users[response.meta["user_id"]] = (
            response.css("#up_user h2::text").extract_first().strip()
        )
        for posting in response.css(".posting"):
            il = FeedEntryItemLoader(
                selector=posting,
                base_url="https://www.{}".format(self.name),
                change_tags={"span": "p"},
            )
            il.add_css("title", ".text strong::text")
            il.add_css("link", '.text a::attr("href")')
            il.add_value(
                "updated",
                datetime.utcfromtimestamp(
                    int(posting.css('.date::attr("data-timestamp")').extract_first())
                    / 1000
                ),
            )
            il.add_css("content_html", ".text span")
            il.add_css("content_html", ".article h4")
            il.add_value("path", response.meta["path"])
            yield il.load_item()



import pytest

from feeds.loaders import build_tree, flatten_tree, serialize_tree
from feeds.tests.utils import load_file, load_fixtures


@pytest.mark.parametrize("in_html_file,out_html_file", load_fixtures("flatten_tree"))
def test_flatten_tree(in_html_file, out_html_file):
    tree = build_tree(load_file(in_html_file))[0]
    flatten_tree(tree)
    assert serialize_tree(tree) == load_file(out_html_file)



from glob import glob


def load_file(filename):
    with open(filename) as f:
        return f.read().strip()


def load_fixtures(test_name):
    path = "feeds/tests/fixtures/{test_name}/*.{direction}.html"
    in_files = sorted(glob(path.format(test_name=test_name, direction="in")))
    out_files = sorted(glob(path.format(test_name=test_name, direction="out")))
    return zip(in_files, out_files)



import re
from collections import OrderedDict
from datetime import timedelta

import scrapy
from dateutil.parser import parse as dateutil_parse
from scrapy.loader.processors import TakeFirst

from feeds.loaders import FeedEntryItemLoader
from feeds.spiders import FeedsXMLFeedSpider


def _remove_empty_headings(text, level=1):
    """
    Recursively remove headings that are not followed by text.
    """
    if not text:
        return

    heading = '<p class="Cat{}HL">'.format(level)
    # Include text of current heading.
    text_new = [text.split(heading)[0]]
    for content in _split_categories(text, heading):
        # Append text of subheadings.
        content = _remove_empty_headings(content, level + 1)
        if content:
            text_new.append(content)
    text = "".join(text_new)

    # Check if text contains content, i.e. not only a heading or tags.
    content = re.sub(r'<p class="Cat\dHL">[^<]+</p>', "", text)
    content = re.sub(r"</?[^>]*>", "", content).strip()
    if not content:
        return None

    return text


def _split_categories(text, heading):
    """
    Splits the text along the given heading. Text before the first heading is
    ignored. The yielded text includes the heading.
    """
    text_new = []
    after_heading = False
    for line in text.splitlines(True):
        if heading in line:
            if after_heading:
                # New heading detected. Yield what we found until now.
                yield "".join(text_new)
                text_new = []
            else:
                # New heading detected and it's the first one.
                after_heading = True
        if after_heading:
            text_new.append(line)
    yield "".join(text_new)


class LwnNetSpider(FeedsXMLFeedSpider):
    name = "lwn.net"
    namespaces = [
        ("dc", "http://purl.org/dc/elements/1.1/"),
        # Default (empty) namespaces are not supported so we just come up with
        # one.
        ("rss", "http://purl.org/rss/1.0/"),
    ]
    itertag = "rss:item"
    # Use XML iterator instead of regex magic which would fail due to the
    # introduced rss namespace prefix.
    iterator = "xml"
    # lwn.net doesn't like it (i.e. blocks us) if we impose too much load.
    custom_settings = {"DOWNLOAD_DELAY": 1.0, "COOKIES_ENABLED": True}

    _subscribed = False

    def start_requests(self):
        if not self.settings.get("HTTPCACHE_ENABLED"):
            self.logger.error("LWN.net spider requires caching to be enabled.")
            return

        username = self.settings.get("FEEDS_SPIDER_LWN_NET_USERNAME")
        password = self.settings.get("FEEDS_SPIDER_LWN_NET_PASSWORD")
        if username and password:
            yield scrapy.FormRequest(
                url="https://{}/login".format(self.name),
                formdata=OrderedDict(
                    [
                        ("Username", username),
                        ("Password", password),
                        ("target", "/MyAccount/"),
                        ("submit", "Log+in"),
                    ]
                ),
                callback=self._after_login,
                # Session cookie is valid for a month. 14 days is a good compromise.
                meta={"cache_expires": timedelta(days=14)},
            )
        else:
            # Username, password or section not found in feeds.cfg.
            self.logger.info(
                "Login failed: No username or password given. "
                "Only free articles are available in full text."
            )
            yield self._start_requests()

    def _after_login(self, response):
        error = response.css(".ErrorMessage::text").extract_first()
        if error:
            self.logger.error("Login failed: {}".format(error))
        else:
            text = "".join(response.css(".ArticleText ::text").extract())
            self._subscribed = "You are currently subscribed" in text
            if not self._subscribed:
                self.logger.warning("You are not subscribed to LWN.net")
        return self._start_requests()

    def _start_requests(self):
        return scrapy.Request(
            "https://{}/headlines/rss".format(self.name),
            self.parse,
            meta={"dont_cache": True},
        )

    def parse_node(self, response, node):
        il = FeedEntryItemLoader(
            response=response, base_url="https://{}".format(self.name)
        )
        updated = dateutil_parse(node.xpath("dc:date/text()").extract_first())
        il.add_value("updated", updated)
        title = node.xpath("rss:title/text()").extract_first()
        paywalled = title.startswith("[$]")
        if paywalled:
            title = title.replace("[$] ", "")
            il.add_value("category", "paywalled")
        link = node.xpath("rss:link/text()").extract_first()
        link = link.replace("rss", "")
        link = link.replace("http://", "https://")
        meta = {"il": il}
        if paywalled and not self._subscribed:
            il.add_value("title", title)
            il.add_value("author_name", node.xpath("dc:creator/text()").extract_first())
            il.add_value(
                "content_text", node.xpath("rss:description/text()").extract_first()
            )
            il.add_value("link", link)
            return il.load_item()
        else:
            if "LWN.net Weekly Edition for" in title:
                meta["updated"] = updated
                callback = self._parse_weekly_edition
                link += "bigpage"
            else:
                callback = self._parse_article
            # Don't include link yet, we will use the subscriber link later.
            # So subscriber articles can be shared from the feed reader and
            # read in browser without logging in.
            return scrapy.Request(link, callback, meta=meta)

    def _parse_article(self, response):
        remove_elems = [
            ".FeatureByline",
            ".GAByline",
            ".Form",
            "form",
            ".MakeALink",
            "br",
        ]
        change_tags = {"div.BigQuote": "blockquote"}
        il = FeedEntryItemLoader(
            response=response,
            parent=response.meta["il"],
            remove_elems=remove_elems,
            change_tags=change_tags,
            base_url="https://{}".format(self.name),
        )
        text = response.css(".ArticleText").extract_first()
        # Remove 'Log in to post comments'.
        text = re.sub(
            r'<hr width="60%" align="left">.*to post comments\)', "", text, flags=re.S
        )
        il.add_css("title", "h1::text")
        il.add_value("content_html", text)
        il.add_css("author_name", ".FeatureByline b ::text")
        il.add_css("author_name", ".GAByline a ::text")
        il.add_css(
            "author_name",
            ".GAByline p ::text",
            re="This article was contributed by (.*)",
        )
        il.add_xpath(
            "updated",
            '//div[@class="FeatureByline"]/text()[preceding-sibling::br]',
            TakeFirst(),
        )
        il.add_xpath("updated", '//div[@class="GAByline"]/p[1]/text()')
        # Last resort if date cannot be extracted and it's a weekly edition.
        if "updated" in response.meta:
            il.add_value("updated", response.meta["updated"])
        if response.css(".MakeALink"):
            # Get subscriber link for paywalled content.
            return scrapy.FormRequest.from_response(
                response,
                formcss=".MakeALink form",
                callback=self._subscriber_link,
                meta={"il": il},
            )
        else:
            il.add_value("link", response.url)
            return il.load_item()

    def _subscriber_link(self, response):
        il = response.meta["il"]
        link = response.css(".ArticleText li a::attr(href)").extract_first()
        il.add_value("link", link)
        return il.load_item()

    def _parse_weekly_edition(self, response):
        remove_elems = ["h1"]
        change_tags = {
            ".Cat1HL": "h1",
            ".Cat2HL": "h2",
            ".Cat3HL": "h3",
            ".SummaryHL": "h4",
        }
        il = FeedEntryItemLoader(
            response=response,
            parent=response.meta["il"],
            change_tags=change_tags,
            remove_elems=remove_elems,
            base_url="https://{}".format(self.name),
        )

        for url in response.css("h2.SummaryHL a::attr(href)").extract():
            yield scrapy.Request(
                response.urljoin(url),
                self._parse_article,
                meta={"il": None, "updated": response.meta["updated"]},
            )

        # Remove articles that have their own page.
        text = []
        in_article = False
        for line in response.css(".ArticleText").extract_first().splitlines(True):
            # Beginning of article.
            if '<h2 class="SummaryHL"><a href="/Articles/' in line:
                in_article = True
            if not in_article:
                text.append(line)
            # End of article. Note that the links to the comments doesn't
            # always include "#comments" so we can't check for that.
            if '">Comments (' in line:
                in_article = False
        text = "".join(text)

        # Remove page editor.
        text = re.sub(r"<b>Page editor</b>: .*", "", text)

        # Recursively remove headings with no content.
        text = _remove_empty_headings(text)

        il.add_css("title", "h1::text")
        il.add_value("content_html", text)
        il.add_value("link", response.url)
        yield il.load_item()



import io
import itertools
from urllib.parse import urljoin, urlparse

import feedparser
import readability.readability
import scrapy
from readability.readability import Document, Unparseable

from feeds.loaders import FeedEntryItemLoader
from feeds.spiders import FeedsSpider
from feeds.utils import generate_feed_header

# Readability's output is not that interesting to justify log level "INFO".
readability.readability.log.info = readability.readability.log.debug


class GenericSpider(FeedsSpider):
    name = "generic"

    def start_requests(self):
        urls = self.settings.get("FEEDS_SPIDER_GENERIC_URLS") or ""
        fulltext_urls = self.settings.get("FEEDS_SPIDER_GENERIC_FULLTEXT_URLS") or ""
        if not urls and not fulltext_urls:
            self.logger.error("Please specify url(s) in the config file!")
            return

        for url, fulltext in itertools.chain(
            zip(urls.split(), itertools.repeat(False)),
            zip(fulltext_urls.split(), itertools.repeat(True)),
        ):
            yield scrapy.Request(
                url, meta={"dont_cache": True, "fulltext": fulltext, "path": url}
            )

    def feed_headers(self):
        return []

    def parse(self, response):
        feed = feedparser.parse(io.BytesIO(response.body))
        if "entries" not in feed or not feed["entries"]:
            self.logger.error("Feed {} contains no entries!".format(response.url))
            return
        feed_entries = feed["entries"]
        feed = feed["feed"]
        yield generate_feed_header(
            title=feed.get("title"),
            subtitle=feed.get("subtitle"),
            link=feed.get("link") or response.url,
            path=response.meta["path"],
            author_name=feed.get("author_detail", {}).get("name"),
            logo=feed.get("image", {}).get("href"),
        )
        base_url = "://".join(urlparse(response.url)[:2])
        for entry in feed_entries:
            # Deals with protocol-relative URLs.
            link = urljoin(base_url, entry["link"])
            il = FeedEntryItemLoader(base_url=base_url)
            il.add_value("path", response.meta["path"])
            il.add_value("updated", entry.get("updated") or entry.get("published"))
            il.add_value("author_name", entry.get("author_detail", {}).get("name"))
            il.add_value("link", link)
            il.add_value("category", [t["term"] for t in entry.get("tags", [])])
            if response.meta["fulltext"]:
                il.add_value("title", entry["title"])
                il.add_value("content_html", entry["content"][0]["value"])
                yield il.load_item()
            else:
                # Content is not part of the feed, scrape it.
                yield scrapy.Request(
                    link, self._parse_article, meta={"feed_entry": entry, "il": il}
                )

    def _parse_article(self, response):
        feed_entry = response.meta["feed_entry"]

        il = FeedEntryItemLoader(parent=response.meta["il"])
        try:
            response.text
        except AttributeError:
            # Response is not text (e.g. PDF, ...).
            il.add_value("title", feed_entry.get("title"))
            il.add_value("content_html", feed_entry.get("summary"))
            return il.load_item()

        doc = Document(response.text, url=response.url)
        il.add_value("title", doc.short_title() or feed_entry.get("title"))
        summary = feed_entry.get("summary")
        try:
            content = doc.summary(html_partial=True)
            if summary and len(summary) > len(content):
                # Something probably went wrong if the extracted content is shorter than
                # the summary.
                raise Unparseable
        except Unparseable:
            content = summary
        il.add_value("content_html", content)

        return il.load_item()



import logging
import os

from xdg.BaseDirectory import save_cache_path, xdg_config_home

# Default settings for Feeds specific configurations.
FEEDS_CONFIG_OUTPUT_PATH = "output"
FEEDS_CONFIG_FILE = os.path.join(xdg_config_home, "feeds.cfg")
FEEDS_CONFIG_CACHE_EXPIRES = 90

# Low level settings intended for scrapy.
# Please use feeds.cfg to configure feeds.

BOT_NAME = "feeds"
SPIDER_MODULES = ["feeds.spiders"]
NEWSPIDER_MODULE = "feeds.spiders"

# Don't overwhelm sites with requests.
CONCURRENT_REQUESTS_PER_DOMAIN = 2
DOWNLOAD_DELAY = 0.25

# Disable telnet
TELNETCONSOLE_ENABLED = False

# Custom item pipeline
ITEM_PIPELINES = {
    "feeds.pipelines.AtomAutogenerateFieldsPipeline": 100,
    "feeds.pipelines.AtomCheckRequiredFieldsPipeline": 110,
    "feeds.pipelines.AtomExportPipeline": 400,
}

SPIDER_MIDDLEWARES = {
    "feeds.spidermiddlewares.FeedsHttpErrorMiddleware": 51,
    "feeds.spidermiddlewares.FeedsHttpCacheMiddleware": 1000,
}

DOWNLOADER_MIDDLEWARES = {
    "feeds.downloadermiddlewares.FeedsHttpCacheMiddleware": 900,
    "scrapy.downloadermiddlewares.httpcache.HttpCacheMiddleware": None,
}

HTTPCACHE_ENABLED = True
HTTPCACHE_STORAGE = "feeds.cache.FeedsCacheStorage"
HTTPCACHE_POLICY = "feeds.cache.FeedsCachePolicy"
HTTPCACHE_DIR = save_cache_path("feeds")
HTTPCACHE_EXPIRATION_SECS = FEEDS_CONFIG_CACHE_EXPIRES * 24 * 60 * 60
HTTPCACHE_IGNORE_HTTP_CODES = list(range(400, 600))

# Do not enable cookies by default to make better use of the cache.
COOKIES_ENABLED = False

RETRY_ENABLED = True
# equals 5 requests in total
RETRY_TIMES = 4

# Don't filter duplicates.
# Spiders sometimes produce feeds with potentially overlapping items.
DUPEFILTER_CLASS = "scrapy.dupefilters.BaseDupeFilter"

# Default user agent. Can be overriden in feeds.cfg.
USER_AGENT = "feeds (+https://github.com/pyfeeds/pyfeeds)"

# Set default level to info.
# Can be overriden with --loglevel parameter.
LOG_LEVEL = logging.INFO

# Stats collection is disabled by default.
# Can be overriden with --stats parameter.
STATS_CLASS = "scrapy.statscollectors.DummyStatsCollector"



import json
from datetime import datetime

import scrapy

from feeds.loaders import FeedEntryItemLoader
from feeds.spiders import FeedsSpider


class IndieHackersComSpider(FeedsSpider):
    name = "indiehackers.com"

    feed_title = "Indie Hackers"
    feed_link = f"https://www.{name}"
    feed_logo = f"{feed_link}/images/favicons/favicon--192x192.png"
    feed_icon = f"{feed_link}/images/favicons/favicon--16x16.png"

    custom_settings = {"DOWNLOAD_DELAY": 1.0}

    def start_requests(self):
        yield scrapy.Request(
            "https://n86t1r3owz-1.algolianet.com/1/indexes/*/queries?"
            + "x-algolia-agent=Algolia%20for%20JavaScript%20(3.35.1)%3B%20Browser"
            + "%20(lite)&x-algolia-application-id=N86T1R3OWZ&"
            + "x-algolia-api-key=5140dac5e87f47346abbda1a34ee70c3",
            method="POST",
            body=(
                '{"requests":[{"indexName":"interviews_publishedAt_desc",'
                + '"params":"query=&page=0&hitsPerPage=20"}]}'
            ),
            meta={"dont_cache": True},
        )

    def parse(self, response):
        response = json.loads(response.text)
        interviews = response["results"][0]["hits"]
        for interview in interviews:
            yield scrapy.Request(
                f"{self.feed_link}/interview/{interview['interviewId']}",
                self._parse_interview,
                meta={
                    "categories": interview["_tags"],
                    "updated": interview["publishedAt"],
                },
            )

    def _parse_interview(self, response):
        remove_elems = [
            ".shareable-quote",
            ".share-bar",
            # Remove the last two h2s and all paragraphs below.
            ".interview-body > h2:last-of-type ~ p",
            ".interview-body > h2:last-of-type",
            ".interview-body > h2:last-of-type ~ p",
            ".interview-body > h2:last-of-type",
        ]
        il = FeedEntryItemLoader(
            timezone="UTC",
            response=response,
            base_url=self.feed_link,
            remove_elems=remove_elems,
        )
        il.add_value("link", response.url)
        il.add_css("title", "h1::text")
        il.add_css("author_name", "header .user-link__name::text")
        il.add_css("content_html", ".interview-body")
        il.add_value(
            "updated", datetime.utcfromtimestamp(response.meta["updated"] / 1000)
        )
        return il.load_item()



from datetime import datetime, timedelta

import scrapy
from dateutil.tz import gettz

from feeds.loaders import FeedEntryItemLoader
from feeds.spiders import FeedsXMLFeedSpider


class ProfilAtSpider(FeedsXMLFeedSpider):
    name = "profil.at"
    namespaces = [
        ("i", "http://www.google.com/schemas/sitemap-image/1.1"),
        ("rss", "http://www.sitemaps.org/schemas/sitemap/0.9"),
    ]
    itertag = "rss:url"
    iterator = "xml"

    feed_title = "PROFIL"
    feed_subtitle = "Österreichs unabhängiges Nachrichtenmagazin"

    def start_requests(self):
        # Scrape this and last month so that the feed is not empty on the first day of a
        # new month.
        this_month = datetime.now(gettz("Europe/Vienna")).date().replace(day=1)
        last_month = (this_month - timedelta(days=1)).replace(day=1)
        for month in [this_month, last_month]:
            yield scrapy.Request(
                "https://www.{}/sitemap-articles-{}.xml".format(
                    self.name, month.strftime("%Y-%m")
                ),
                meta={"dont_cache": True, "handle_httpstatus_list": [404]},
            )

    def parse_node(self, response, node):
        url = node.xpath("rss:loc/text()").extract_first()
        updated = node.xpath("rss:lastmod/text()").extract_first()
        return scrapy.Request(url, self.parse_item, meta={"updated": updated})

    def parse_item(self, response):
        remove_elems = [
            "aside",
            "script",
            "h1",
            "source",
            ".breadcrumbs",
            ".author-date",
            ".artikel-social-kommentar",
            ".bild-copyright",
            ".ressortTitleMobile",
            ".article-number",
            ".artikel-kommentarlink",
            ".umfrage-wrapper",
            ".articleIssueInfo",
        ]
        il = FeedEntryItemLoader(
            response=response,
            base_url="https://{}".format(self.name),
            remove_elems=remove_elems,
        )
        il.add_value("link", response.url)
        author_name = (
            response.css(".author-date ::text").re(r"(?:Von)?\s*(\w+ \w+)") or "Red."
        )
        il.add_value("author_name", author_name)
        il.add_css("title", 'h1[itemprop="headline"]::text')
        il.add_value("updated", response.meta["updated"])
        il.add_css("content_html", "article")
        return il.load_item()



from datetime import timezone

import scrapy


def to_rfc3339(date_time):
    """Return RFC 3339 representation of date_time with timezone UTC."""
    # Shifting to UTC and hardcoding "Z" for the timezone is a cheap way to get a
    # RFC 3339 compliant timestamp.
    return date_time.astimezone(timezone.utc).strftime("%Y-%m-%dT%H:%M:%SZ")


class BaseItem(scrapy.Item):
    # Required
    # The feed/entry id. It may be auto generated in case a link is present.
    id = scrapy.Field()

    # The feed/entry title.
    title = scrapy.Field()

    # The last updated date of the feed.
    # Required date format: RFC 3339 (ISO 8601 extended format)
    updated = scrapy.Field(serializer=to_rfc3339)

    # Recommended
    author_name = scrapy.Field()
    author_email = scrapy.Field()

    # A unique link to a site/item.
    link = scrapy.Field()

    # Optional
    category = scrapy.Field()

    # Optional
    path = scrapy.Field()


class FeedItem(BaseItem):
    # Optional
    subtitle = scrapy.Field()

    # Optional
    icon = scrapy.Field()

    # Optional
    logo = scrapy.Field()

    # Optional
    link_self = scrapy.Field()


class FeedEntryItem(BaseItem):
    # Recommended
    content_text = scrapy.Field()
    content_html = scrapy.Field()

    # Optional
    enclosure = scrapy.Field()



from urllib.parse import urlsplit

import scrapy

from feeds.loaders import FeedEntryItemLoader
from feeds.spiders import FeedsXMLFeedSpider


class TheOatmealComSpider(FeedsXMLFeedSpider):
    name = "theoatmeal.com"
    start_urls = ["http://theoatmeal.com/feed/rss"]

    namespaces = [
        ("dc", "http://purl.org/dc/elements/1.1/"),
        ("def", "http://purl.org/rss/1.0/"),
    ]
    iterator = "xml"
    itertag = "def:item"

    feed_title = "The Oatmeal"
    feed_subtitle = (
        "The oatmeal tastes better than stale skittles found under the couch cushions"
    )
    _base_url = "https://{}".format(name)
    feed_icon = "https://{}/favicon.ico".format(name)
    feed_logo = (
        "http://s3.amazonaws.com/theoatmeal-img/default/header2016/logo_rainbow.png"
    )

    def parse_node(self, response, node):
        url = node.xpath("def:link/text()").extract_first()
        author_name = node.xpath("dc:creator/text()").extract_first()
        updated = node.xpath("dc:date/text()").extract_first()
        return scrapy.Request(
            url, self.parse_item, meta={"updated": updated, "author_name": author_name}
        )

    def parse_item(self, response):
        il = FeedEntryItemLoader(response=response, base_url=self._base_url)
        il.add_value("updated", response.meta["updated"])
        il.add_value("author_name", response.meta["author_name"])
        il.add_value("link", response.url)
        il.add_css("title", "title::text", re="(.*) - The Oatmeal")
        il.add_value("category", urlsplit(response.url).path.strip("/").split("/")[0])

        # comics
        il.add_css("content_html", "#comic > img")
        il.add_css("content_html", "#comic > p > img")

        # blog
        il.add_css("content_html", "#blog .center_text img")
        return il.load_item()



import json
import re

import scrapy

from feeds.loaders import FeedEntryItemLoader
from feeds.spiders import FeedsSpider


class Pusl4ComSpider(FeedsSpider):
    name = "puls4.com"
    start_urls = ["https://www.puls4.com/api/json-fe/page/sendungen"]

    feed_icon = (
        "https://www.puls4.com/bundles/wundermanpuls4/images/" + "favicon/favicon.png"
    )

    def parse(self, response):
        path = json.loads(response.text)["content"][0]["url"]
        return scrapy.Request(
            response.urljoin(path), self._parse_shows_list, meta={"dont_cache": True}
        )

    def _parse_shows_list(self, response):
        shows = json.loads(response.text)["formatOverviewItems"]
        for show in shows:
            time = re.findall(r"(\d{2}:\d{2})", show["announcement"]) or None
            if time:
                time = time[0]
            yield scrapy.Request(
                response.urljoin(show["channelUrl"] + "/videos/Ganze-Folgen"),
                self._parse_show,
                meta={
                    "dont_cache": True,
                    "time": time,
                    "handle_httpstatus_list": [404],
                },
            )

    def _parse_show(self, response):
        if response.status != 200:
            self.logger.debug("Ignoring response with status code != 200")
            return

        urls = response.css("a.media-preview-link::attr(href)").extract()[:3]
        for url in urls:
            yield scrapy.Request(
                response.urljoin(url),
                self._parse_episode,
                meta={"time": response.meta["time"]},
            )

    def _parse_episode(self, response):
        il = FeedEntryItemLoader(
            response=response,
            base_url="https://{}".format(self.name),
            timezone="Europe/Vienna",
            dayfirst=True,
        )
        il.add_value("link", response.url)
        il.add_xpath(
            "title",
            '//meta[@name="title"]/@content',
            re=r"(?s)(.*?)(?: vom .*)? - puls4\.com",
        )
        il.add_value(
            "updated",
            "{} {}".format(
                response.xpath('//meta[@name="title"]/@content').re_first(
                    r".*vom (\d{2}\.\d{2}\.\d{4}).*"
                ),
                response.meta["time"] or "00:00",
            ),
        )
        il.add_value(
            "content_html",
            '<img src="{}">'.format(
                response.xpath('//meta[@property="og:image"]/@content').extract_first()
            ),
        )
        il.add_css("content_html", ".player-video-description-intro::text")
        return il.load_item()



import scrapy

from feeds.loaders import FeedEntryItemLoader
from feeds.spiders import FeedsXMLFeedSpider
from feeds.utils import generate_feed_header


class FtComSpider(FeedsXMLFeedSpider):
    name = "ft.com"
    custom_settings = {"REFERER_ENABLED": False}

    _titles = {}

    def start_requests(self):
        self._ressorts = self.settings.get("FEEDS_SPIDER_FT_COM_RESSORTS", [])
        if self._ressorts:
            self._ressorts = set(self._ressorts.split())
        else:
            self._ressorts = set(["homepage"])

        for ressort in self._ressorts:
            if ressort == "homepage":
                path = ""
            else:
                path = ressort
            yield scrapy.Request(
                "https://www.{}/{}?format=rss".format(self.name, path),
                meta={"dont_cache": True, "ressort": ressort},
            )

    def parse_node(self, response, node):
        if not self._titles.get(response.meta["ressort"]):
            self._titles[response.meta["ressort"]] = response.xpath(
                "//channel/title/text()"
            ).extract_first()

        url = node.xpath("link/text()").extract_first()
        return scrapy.Request(
            url,
            self._parse_article,
            meta={"ressort": response.meta["ressort"]},
            headers={"Referer": "https://www.facebook.com"},
        )

    def feed_headers(self):
        for ressort in self._ressorts:
            yield generate_feed_header(
                title="Financial Times › {}".format(self._titles.get(ressort, ressort)),
                link="https://www.{}".format(self.name),
                icon=(
                    "https://www.{}/__origami/service/image/v2/images/raw/"
                    + "ftlogo-v1%3Abrand-ft-logo-square-coloured?source=update-logos"
                    + "&width=32&height=32&format=png"
                ).format(self.name),
                logo=(
                    "https://www.{}/__origami/service/image/v2/images/raw/"
                    + "ftlogo-v1%3Abrand-ft-logo-square-coloured?source=update-logos"
                    + "&width=194&height=194&format=png"
                ).format(self.name),
                path=ressort,
            )

    def _parse_article(self, response):
        remove_elems = [".n-content-recommended--single-story"]
        change_tags = {".topper__standfirst": "h2"}
        il = FeedEntryItemLoader(
            response=response,
            base_url="https://{}".format(self.name),
            remove_elems=remove_elems,
            change_tags=change_tags,
        )
        il.add_value("link", response.url)
        il.add_css("title", 'meta[property="og:title"]::attr(content)')
        il.add_css("author_name", 'meta[property="article:author"]::attr(content)')
        il.add_css("updated", 'meta[property="article:modified_time"]::attr(content)')
        il.add_css("content_html", ".topper__standfirst")
        il.add_css("content_html", ".article__content-body")
        il.add_css("category", ".n-content-ta ::text")
        il.add_value("path", response.meta["ressort"])
        return il.load_item()



import scrapy

from feeds.loaders import FeedEntryItemLoader
from feeds.spiders import FeedsSpider


class AkCiandoComSpider(FeedsSpider):
    name = "ak.ciando.com"
    start_urls = [
        "https://ak.ciando.com/shop/index.cfm?fuseaction=cat_overview&cat_ID=0"
        "&cat_nav=0&more_new=1&rows=100&intStartRow=1"
    ]

    feed_title = "AK Digitale Bibliothek"
    feed_subtitle = "Neue Titel in der digitalen AK Bibliothek"
    feed_link = "https://{}".format(name)

    def parse(self, response):
        for link in response.xpath('//p[@class="p_blr_title"]//a/@href').extract():
            yield scrapy.Request(response.urljoin(link), self.parse_item)

    def parse_item(self, response):
        il = FeedEntryItemLoader(
            selector=response.xpath('//div[@id="maincontentbook"]'),
            base_url=self.feed_link,
        )
        il.add_xpath("title", '//h1[@class="p_book_title"]/text()')
        il.add_xpath("title", '//h3[@class="p_book_title_ebook"]/text()')
        il.add_value("link", response.url)
        il.add_value("author_name", self.feed_title)
        il.add_xpath("content_html", '//h1[@class="p_book_title"]/text()')
        il.add_xpath("content_html", '//h2[@class="p_book_author"]/text()')
        il.add_xpath("content_html", '//p[@class="p_book_publisher"]/text()')
        il.add_xpath("content_html", '//p[@class="p_book_isbn"]/text()')
        il.add_xpath("content_html", '(//span[@class="txt10px"])[1]/text()')
        il.add_xpath("content_html", '(//span[@class="txt10px"])[3]/text()')
        il.add_xpath("content_html", '//div[@class="bookcontent"]//text()')
        il.add_xpath("content_html", '//div[@class="p_book_image"]/img')
        il.add_xpath("content_html", '//span[@style="color:red;"]/b/text()')
        return il.load_item()



from urllib.parse import urljoin

import scrapy

from feeds.loaders import FeedEntryItemLoader
from feeds.spiders import FeedsSpider
from feeds.utils import generate_feed_header


class FlimmitComSpider(FeedsSpider):
    name = "flimmit.com"

    _base_url = "https://www.{}".format(name)
    _categories = ["filme", "serien", "europa", "kinder"]

    def start_requests(self):
        categories = self.settings.get("FEEDS_SPIDER_FLIMMIT_COM_CATEGORIES")
        if categories:
            self._categories = categories.split()

        for category in self._categories:
            yield scrapy.Request(
                urljoin(
                    self._base_url,
                    "https://www.flimmit.com/{}/?dir=desc&order=news_from_date".format(
                        category
                    ),
                ),
                meta={"dont_cache": True, "path": category},
            )

    def feed_headers(self):
        for category in self._categories:
            yield generate_feed_header(
                title="Flimmit.com: {}".format(category.title()),
                link="https://www.{}".format(self.name),
                icon="https://www.{}/favicon.ico".format(self.name),
                path=category,
            )

    @staticmethod
    def _fix_img_src(elem):
        if "data-original" in elem.attrib:
            elem.attrib["src"] = elem.attrib["data-original"]
        return elem

    def parse(self, response):
        for link in response.css(
            ".category-products .item .product-image::attr('href')"
        ).extract():
            yield scrapy.Request(
                link, self._parse_item, meta={"path": response.meta["path"]}
            )

    @staticmethod
    def _create_attributes_html(keys, values):
        attr_html = "<ul>"
        for key, value in zip(keys, values):
            attr_html += "<li><strong>{key}</strong> {value}</li>".format(
                key=key, value=value
            )
        attr_html += "</ul>"
        return attr_html

    def _parse_item(self, response):
        if response.css(".product-package"):
            yield from self.parse(response)
        else:
            remove_elems = [
                "h1",
                "h6",
                "img[src*='overlay-europa']",
                ".product-attr-useraction",
                ".product-award",
                ".product-image",
                ".bundle-seasons",
            ]
            replace_elems = {"img": self._fix_img_src}
            change_tags = {"div.attr-text": "strong"}

            il = FeedEntryItemLoader(
                response=response,
                base_url=self._base_url,
                remove_elems=remove_elems,
                replace_elems=replace_elems,
                change_tags=change_tags,
            )

            il.add_css("title", "h1 ::text")
            il.add_value("link", response.url)
            il.add_value("path", response.meta["path"])

            il.add_css("content_html", ".product-description .product-image img")
            attributes = self._create_attributes_html(
                [
                    e.strip()
                    for e in response.css(
                        ".product-attributes .col-xs-5 ::text"
                    ).extract()
                    if e.strip()
                ],
                [
                    e.strip()
                    for e in response.css(
                        ".product-attributes .col-xs-7 ::text"
                    ).extract()
                    if e.strip()
                ],
            )
            il.add_value("content_html", attributes)
            il.add_css("content_html", ".product-info .item")

            if (
                "abo"
                in response.css(
                    ".product-description img::attr('data-overlay')"
                ).extract_first()
            ):
                il.add_value("category", "abo")
            il.add_css("category", "span[itemprop='genre']::text")
            # Kollektionen
            il.add_css(
                "category",
                ".product-info .item > div.attr-text:last-of-type ~ a ::text",
            )

            yield il.load_item()



import json
import re
from urllib.parse import urlparse

import lxml
import scrapy
from inline_requests import inline_requests
from scrapy.selector import Selector

from feeds.loaders import FeedEntryItemLoader
from feeds.spiders import FeedsXMLFeedSpider
from feeds.utils import generate_feed_header


class OrfAtSpider(FeedsXMLFeedSpider):
    name = "orf.at"
    namespaces = [
        ("dc", "http://purl.org/dc/elements/1.1/"),
        ("orfon", "http://rss.orf.at/1.0/"),
        ("rdf", "http://www.w3.org/1999/02/22-rdf-syntax-ns#"),
        # Default (empty) namespaces are not supported so we just come up with one.
        ("rss", "http://purl.org/rss/1.0/"),
    ]
    itertag = "rss:item"
    # Use XML iterator instead of regex magic which would fail due to the
    # introduced rss namespace prefix.
    iterator = "xml"

    def start_requests(self):
        channels = self.settings.get("FEEDS_SPIDER_ORF_AT_CHANNELS")
        if channels:
            channels = set(channels.split())
        else:
            channels = {"news"}
        available_channels = {
            "burgenland",
            "fm4",
            "help",
            "kaernten",
            "news",
            "noe",
            "oe3",
            "oesterreich",
            "ooe",
            "religion",
            "salzburg",
            "science",
            "sport",
            "steiermark",
            "tirol",
            "vorarlberg",
            "wien",
        }
        unknown_channels = channels - available_channels
        if unknown_channels:
            self.logger.warning(
                "Unknown channel(s) in config file: {}".format(
                    ", ".join(unknown_channels)
                )
            )

        for channel in channels:
            yield scrapy.Request(
                "https://rss.orf.at/{}.xml".format(channel),
                meta={"path": channel, "dont_cache": True},
            )

        self._channels = channels

        self._authors = [
            author
            for author in (
                self.settings.get("FEEDS_SPIDER_ORF_AT_AUTHORS", "").split("\n")
            )
            if author
        ]

    def feed_headers(self):
        for channel in self._channels:
            channel_url = "{}.ORF.at".format(channel)
            yield generate_feed_header(
                title=channel_url,
                link="https://{}".format(channel_url.lower()),
                path=channel,
                logo=self._get_logo(channel),
            )

        for author in self._authors:
            yield generate_feed_header(title="ORF.at: {}".format(author), path=author)

    def parse(self, response):
        selector = Selector(response, type="xml")

        doc = lxml.etree.ElementTree(lxml.etree.fromstring(response.body))
        if doc.getroot().nsmap:
            self._register_namespaces(selector)
            nodes = selector.xpath("//%s" % self.itertag)
        else:
            nodes = selector.xpath("//item")

        return self.parse_nodes(response, nodes)

    def parse_node(self, response, node):
        if "orfon" in node.namespaces:
            return self._parse_extended_node(response, node)
        else:
            return self._parse_simple_node(response, node)

    def _parse_extended_node(self, response, node):
        categories = [
            node.xpath("orfon:storyType/@rdf:resource").re_first("urn:orfon:type:(.*)"),
            node.xpath("dc:subject/text()").extract_first(),
        ]
        substories = node.xpath(
            "orfon:substories/rdf:Bag/rdf:li/@rdf:resource"
        ).extract()
        updated = node.xpath("dc:date/text()").extract_first()
        meta = {
            "path": response.meta["path"],
            "categories": categories,
            "updated": updated,
        }
        if substories:
            links = substories
        else:
            links = [node.xpath("rss:link/text()").extract_first()]
        for link in links:
            fixed_link = self._extract_link(link)
            if fixed_link is None or any(
                fixed_link.startswith(url)
                for url in ["https://debatte.orf.at", "https://iptv.orf.at"]
            ):
                self.logger.debug(
                    "Ignoring link to '{}' ('{}')".format(link, fixed_link)
                )
            else:
                yield scrapy.Request(fixed_link, self._parse_article, meta=meta)

    def _parse_simple_node(self, response, node):
        meta = {
            "path": response.meta["path"],
            "categories": node.xpath("category/text()").extract(),
            "updated": node.xpath("pubDate/text()").extract_first(),
        }
        link = node.xpath("link/text()").extract_first()
        return scrapy.Request(link, self._parse_article, meta=meta)

    @staticmethod
    def _extract_link(link):
        """Extract a working link from a possibly broken link."""
        if link is None:
            return None
        match = re.search(r"https?://(?:[^\.]+\.)?orf\.at/(?:news/)?stories/\d*", link)
        return match.group(0) + "/" if match else None

    @inline_requests
    def _parse_article(self, response):
        # Heuristic for news.ORF.at to to detect teaser articles.
        more = self._extract_link(
            response.css(
                ".story-story p > strong:contains('Mehr') + a::attr(href), "
                + ".story-story p > a:contains('Lesen Sie mehr')::attr(href)"
            ).extract_first()
        )
        if more and more != response.url:
            self.logger.debug("Detected teaser article, redirecting to {}".format(more))
            response = yield scrapy.Request(more, meta=response.meta)

        remove_elems = [
            ".byline",
            "h1",
            ".socialshare",
            ".socialShareWrapper",
            ".socialButtons",
            ".credit",
            ".toplink",
            ".offscreen",
            ".storyMeta",
            "script",
            ".oon-youtube-logo",
            ".vote",
            # redesign
            "#more-to-read-anchor",
            ".social-buttons",
            ".story-horizontal-ad",
            ".linkcard",
            ".geolocation",  # Bundesländer
        ]
        pullup_elems = {
            ".remote .slideshow": 1,
            ".remote .instagram": 1,
            ".remote .facebook": 1,
            ".remote .twitter": 1,
            ".remote .youtube": 1,
            ".remote table": 1,
        }
        replace_elems = {
            ".video": "<p><em>Hinweis: Das eingebettete Video ist nur im Artikel "
            + "verfügbar.</em></p>",
            ".slideshow": (
                "<p><em>Alte Slideshows werden nicht mehr unterstützt.</em></p>"
            ),
        }
        change_attribs = {"img": {"data-src": "src", "srcset": "src"}}
        change_tags = {
            ".image": "figure",
            ".caption": "figcaption",
            ".fact": "blockquote",  # FM4
        }
        author, author_selector = self._extract_author(response)
        if author:
            self.logger.debug("Extracted possible author '{}'".format(author))
            # Remove the paragraph that contains the author.
            remove_elems.insert(0, author_selector)
        else:
            self.logger.debug("Could not extract author name")
            author = "{}.ORF.at".format(response.meta["path"])

        for slideshow in response.css(".slideshow"):
            link = response.urljoin(
                slideshow.css('::attr("data-slideshow-json-href")').extract_first()
            ).replace("jsonp", "json")
            slideshow_id = slideshow.css('::attr("id")').extract_first()
            slideshow_response = yield scrapy.Request(link)
            replace_elems["#{}".format(slideshow_id)] = self._create_slideshow_html(
                slideshow_response
            )

        il = FeedEntryItemLoader(
            response=response,
            remove_elems=remove_elems,
            pullup_elems=pullup_elems,
            replace_elems=replace_elems,
            change_attribs=change_attribs,
            change_tags=change_tags,
        )

        # The field is part of a JSON that is sometimes not valid, so don't bother with
        # parsing it properly.
        match = re.search(r'"datePublished": "([^"]+)"', response.text)
        if match:
            # news.ORF.at
            updated = match.group(1)
        else:
            # other
            updated = response.meta["updated"]
        il.add_value("updated", updated)
        il.add_css("title", ".story-lead-headline ::text")  # news
        il.add_css("title", "#ss-storyText > h1 ::text")  # FM4, science
        il.add_value("link", response.url)
        il.add_css("content_html", ".opener img")  # FM4, news
        il.add_css("content_html", ".story-lead-text")  # news
        il.add_css("content_html", "#ss-storyText")
        il.add_css("content_html", "#ss-storyContent")  # news
        il.add_value("author_name", author)
        if author in self._authors:
            il.add_value("path", author)
        il.add_value("path", response.meta["path"])
        il.add_value("category", response.meta["categories"])
        yield il.load_item()

    @staticmethod
    def _create_slideshow_html(response):
        slideshow = json.loads(response.text)
        figures = []
        for photo in slideshow["photos"]:
            url = photo["url"]
            caption = photo.get("description") or ""
            figures.append(
                (
                    '<figure><div><img src="{url}"></div>'
                    + "<figcaption>{caption}</figcaption></figure>"
                ).format(url=url, caption=caption)
            )
        return "<div>" + "".join(figures) + "</div>"

    @staticmethod
    def _extract_author(response):
        domain = urlparse(response.url).netloc
        if domain == "fm4.orf.at":
            author = (
                response.css("#ss-storyText > .socialButtons")
                .xpath(
                    "following-sibling::p[("
                    + "starts-with(., 'Von') or starts-with(., 'von') "
                    + "or starts-with(., 'By') or starts-with(., 'by')"
                    + ") and position() = 1]/a/text()"
                )
                .extract_first()
            )
            author_selector = "#ss-storyText > .socialButtons + p"
            if author:
                return (author.strip(), author_selector)
        elif domain in ["science.orf.at", "help.orf.at", "religion.orf.at"]:
            try:
                author = (
                    response.css("#ss-storyText > p:not(.date):not(.toplink)::text")
                    .extract()[-1]
                    .strip()
                )
                # Possible author string must be in [2, 50].
                if 2 <= len(author) <= 50:
                    # Only take the author name before ",".
                    author = re.split(r"[/,]", author)[0]
                    return (
                        author.strip(),
                        (
                            "#ss-storyText > p:not(.date):not(.toplink):"
                            + "contains('{}')"
                        ).format(author),
                    )
            except IndexError:
                pass
        else:
            author = response.css(".byline ::text").extract_first()
            if author:
                return (re.split(r"[/,]", author)[0].strip(), ".byline")

        return (None, None)

    @staticmethod
    def _get_logo(channel):
        images = {
            "fm4": ("tube", "fm4"),
            "help": ("tube", "help"),
            "science": ("tube", "science"),
            "news": ("news", "news"),
        }
        return (
            "https://tubestatic.orf.at/mojo/1_3/storyserver/{}/{}/images/"
            + "touch-icon-ipad-retina.png"
        ).format(*images.get(channel, images.get("news")))



import logging
import os

import click
from scrapy.crawler import CrawlerProcess
from scrapy.utils.log import configure_logging
from scrapy.utils.project import get_project_settings
from twisted.python import failure

from feeds.cache import FeedsCache
from feeds.settings import load_feeds_settings

logger = logging.getLogger(__name__)


def run_cleanup_cache(settings):
    cache = FeedsCache(settings)
    cache.cleanup()


def spiders_to_crawl(process, argument_spiders):
    if argument_spiders:
        # Spider(s) given as command line argument(s).
        logger.debug("Using command line to decide what spiders to run.")
        return argument_spiders

    try:
        # Spider(s) given in configuration file.
        spiders = process.settings.get("FEEDS_CONFIG_SPIDERS")
        logger.debug("Using configuration file to decide what spiders to run.")
        if spiders:
            return spiders.split()
        return None
    except KeyError:
        # Load all available spiders.
        logger.debug("All available spiders will be run.")
        return process.spider_loader.list()


@click.group()
@click.option(
    "--loglevel",
    "-l",
    default="info",
    type=click.Choice(["debug", "info", "warning", "error"]),
)
@click.option(
    "--config", "-c", type=click.File(), help="Feeds configuration for feeds."
)
@click.option("--pdb/--no-pdb", default=False, help="Enable pdb on failure.")
@click.pass_context
def cli(ctx, loglevel, config, pdb):
    """
    feeds creates feeds for pages that don't have feeds.
    """
    if pdb:
        failure.startDebugMode()

    # A pip-installed Feeds does not have a scrapy.cfg in its project root.
    os.environ["SCRAPY_SETTINGS_MODULE"] = "feeds.default_settings"

    settings = load_feeds_settings(config)
    settings.set("LOG_LEVEL", loglevel.upper())
    ctx.obj["settings"] = settings


@cli.command()
@click.argument("spiders", nargs=-1)
@click.option("--stats/--no-stats", default=False, help="Output scraping stats.")
@click.pass_context
def crawl(ctx, spiders, stats):
    """
    Crawl one or many or all pages.

    What spider(s) to run is determined in the following order:

      1. Spider(s) given as argument(s)

      2. Spider(s) specified in the configuration file

    Note that if a spider is given as an argument, the spiders in the
    configuration file are ignored. All available spiders will be used to
    crawl if no arguments are given and no spiders are configured.
    """
    settings = ctx.obj["settings"]
    if stats:
        settings.set("STATS_CLASS", "scrapy.statscollectors.MemoryStatsCollector")

    # Start a new crawler process.
    process = CrawlerProcess(settings)
    spiders = spiders_to_crawl(process, spiders)
    if not spiders:
        logger.error("Please specify what spiders you want to run!")
    else:
        for spider in spiders:
            logger.info("Starting crawl of {} ...".format(spider))
            process.crawl(spider)

    process.start()

    if settings.getbool("HTTPCACHE_ENABLED"):
        run_cleanup_cache(settings)


@cli.command()
def list():
    """List all available spiders."""
    settings = get_project_settings()
    settings["LOG_ENABLED"] = False
    process = CrawlerProcess(settings)
    for s in sorted(process.spider_loader.list()):
        print(s)


@cli.command()
@click.pass_context
def cleanup(ctx):
    """
    Cleanup old cache entries.

    By default, entries older than 90 days will be removed. This value can be
    overriden in the config file.
    """
    settings = ctx.obj["settings"]
    # Manually configure logging since we don't have a CrawlerProcess which
    # would take care of that.
    configure_logging(settings)

    if not settings.getbool("HTTPCACHE_ENABLED"):
        logger.error("Cache is disabled, will not clean up cache dir.")
        return 1

    run_cleanup_cache(settings)


def main():
    cli(obj={})



from feeds.loaders import FeedItemLoader


def generate_feed_header(
    title=None,
    subtitle=None,
    link=None,
    path=None,
    author_name=None,
    icon=None,
    logo=None,
):
    il = FeedItemLoader()
    il.add_value("title", title)
    il.add_value("subtitle", subtitle)
    il.add_value("link", link)
    il.add_value("path", path)
    il.add_value("author_name", author_name)
    il.add_value("icon", icon)
    il.add_value("logo", logo)
    return il.load_item()



import re

import scrapy

from feeds.loaders import FeedEntryItemLoader
from feeds.spiders import FeedsXMLFeedSpider


class DieTiwagOrgSpider(FeedsXMLFeedSpider):
    name = "dietiwag.org"
    start_urls = ["http://www.dietiwag.org/rss.xml.php"]

    feed_title = "dietiwag.org"
    feed_subtitle = "die andere seite der tiroler wasser kraft"
    feed_link = "http://www.{}".format(name)
    feed_icon = "http://www.{}/favicon.ico".format(name)

    def parse_node(self, response, node):
        il = FeedEntryItemLoader(selector=node)
        url = node.xpath("link/text()").extract_first()
        il.add_value("link", url)
        il.add_xpath("updated", "pubDate/text()")
        il.add_xpath(
            "title",
            "title/text()",
            # Use re.DOTALL since some titles have newlines in them.
            re=re.compile("(?:Artikel|Tagebuch): (.*)", re.DOTALL),
        )
        return scrapy.Request(url, self._parse_article, meta={"il": il})

    def _parse_article(self, response):
        remove_elems = [
            ".noprint",
            "form",
            "font[size='3'] > b",
            "font[size='2'] > b:first-child",
            'a[href="mailto:m.wilhelm@dietiwag.org"]',
            "br:first-child",
            "br:first-child",
            "br:first-child",
            "br:first-child",
            "br:first-child",
            "br:first-child",
            "br:last-child",
            "br:last-child",
            "br:last-child",
            "br:last-child",
            "br:last-child",
            "br:last-child",
        ]
        replace_regex = {
            r"\[\d{2}\.\d{2}\.\d{4}\]": "",
            # A0 is a non-breaking space in latin1.
            "\xA0": "",
            r"<br>\s*<br>\s*\d{1,2}\.\d{1,2}\.\d{4}\s*<br>": "",
        }
        change_attribs = {"font": {"size": None, "face": None, "color": None}}
        change_tags = {"font": "div", "center": "div"}
        il = FeedEntryItemLoader(
            response=response,
            base_url=response.url,
            remove_elems=remove_elems,
            replace_regex=replace_regex,
            change_attribs=change_attribs,
            change_tags=change_tags,
            parent=response.meta["il"],
        )
        il.add_css("author_name", ".sidebar .authors__name::text")
        if response.css(".printwidth2"):
            il.add_css("content_html", ".printwidth2")
        else:
            # Tagebuch
            il.add_css("content_html", ".lineall")
            il.add_value("category", "Tagebuch")
        return il.load_item()



from scrapy.linkextractors import LinkExtractor
from scrapy.spiders import Rule

from feeds.loaders import FeedEntryItemLoader
from feeds.spiders import FeedsCrawlSpider


class CbirdAtSpider(FeedsCrawlSpider):
    name = "cbird.at"
    allowed_domains = ["cbird.at"]
    start_urls = ["https://cbird.at/hilfe/neu/", "https://cbird.at/impressum"]
    rules = (Rule(LinkExtractor(allow=(r"hilfe/neu/(\d+)",)), callback="parse_item"),)

    feed_title = "Neue cbird Versionen"
    feed_subtitle = "Die neuesten Versionen von cbird."
    feed_link = start_urls[0]
    feed_icon = "https://{}/images/bird-1.png".format(name)
    feed_logo = "https://{}/images/bird-1.png".format(name)

    def parse_item(self, response):
        il = FeedEntryItemLoader(
            selector=response.xpath('//div[@class="main"]'), timezone="Europe/Vienna"
        )
        il.add_xpath("title", "h1/text()")
        il.add_value("link", response.url)
        il.add_xpath("content_html", "h1/following-sibling::*")
        il.add_value("updated", response.url.rstrip("/").split("/")[-1].split("_")[0])
        il.add_value("author_name", self.name)
        return il.load_item()

    def parse_imprint(self, response):
        self._author_name = (
            response.xpath('//div[@class="main"]/p/text()').re_first("Firma (.*)")
            or self.name
        )



import re
from datetime import datetime

import scrapy

from feeds.loaders import FeedEntryItemLoader
from feeds.spiders import FeedsSpider
from feeds.utils import generate_feed_header


class UsenixOrgSpider(FeedsSpider):
    name = "usenix.org"
    start_urls = ["https://www.usenix.org/publications/login"]

    def feed_headers(self):
        return []

    def parse(self, response):
        # Only scrape the last 8 issues.
        issues = response.css(".issues .month a::attr(href)").extract()[:8]
        yield generate_feed_header(
            title=";login:",
            subtitle="The Usenix Magazine",
            link=response.url,
            path="login",
        )
        for issue in issues:
            yield scrapy.Request(response.urljoin(issue), self.parse_login_issue)

    def parse_login_issue(self, response):
        remove_elems = [
            ".field-name-field-file-access",
            ".field-name-field-login-issue-file",
            ".field-name-field-product",
            ".field-commerce-price",
            ".views-field-field-file-access",
            ".view-header",
        ]
        il = FeedEntryItemLoader(
            response=response,
            base_url="https://www.{}".format(self.name),
            remove_elems=remove_elems,
        )
        il.add_value("link", response.url)
        title = response.css("h1::text").extract_first().strip()
        il.add_value("title", title)
        il.add_value("updated", self._date_from_title(title))
        il.add_css("content_html", ".content-wrapper")
        il.add_value("path", "login")
        if response.css(".usenix-files-protected"):
            il.add_value("category", "paywalled")
        return il.load_item()

    def _date_from_title(self, issue):
        """Try to guess the publication date of an issue from the title."""
        match = re.search(
            r"(?P<season>Spring|Summer|Fall|Winter) (?P<year>\d{4})", issue
        )
        if match:
            seasons = {"Winter": "1", "Spring": "4", "Summer": "7", "Fall": "10"}
            month = int(seasons[match.group("season")])
            year = int(match.group("year"))
            date = datetime(day=1, month=month, year=year)
            # Issues become free after a year which should be reflected by
            # bumping the updated date by a year as well.
            date_free = datetime(day=1, month=month, year=year + 1)
            return date_free if date_free < date.utcnow() else date
        else:
            self.logger.warning('Could not extract date from title "{}"!'.format(issue))



import scrapy

from feeds.loaders import FeedEntryItemLoader
from feeds.spiders import FeedsSpider


class BibliowebAtSpider(FeedsSpider):
    name = "biblioweb.at"
    custom_settings = {
        "DUPEFILTER_CLASS": "scrapy.dupefilters.RFPDupeFilter",
        "COOKIES_ENABLED": True,
    }

    _days = 60

    def start_requests(self):
        self._library = self.settings.get(
            "FEEDS_SPIDER_BIBLIOWEB_AT_LOCATION", ""
        ).lower()
        if self._library:
            self.feed_path = self._library
            self.feed_title = "Bibliothek {}".format(self._library.title())
            self.feed_subtitle = "Neue Titel in der {}".format(self.feed_title)
            self.feed_link = "https://www.biblioweb.at/{}/".format(self._library)
            yield scrapy.Request(
                "https://www.biblioweb.at/{}/start.asp".format(self._library),
                callback=self.parse,
                meta={"dont_cache": True},
            )
        else:
            # Key location or section biblioweb.at not found in feeds.cfg.
            self.logger.error(
                "A location is required for spider '{name}'. Please add a "
                "configuration block for '{name}' with the 'location' you "
                "want to scrape.".format(name=self.name)
            )

    def parse(self, response):
        # Ignore the initial response to start.asp as it is required to get the
        # ASP cookie. Without this cookie the requests to webopac123 (!) are
        # ignored and will be redirected to the "login" page.
        return scrapy.Request(
            "https://www.biblioweb.at/webopac123/webopac.asp"
            "?kat=1&content=show_new&seit={}&order_by=Sachtitel".format(self._days),
            callback=self.parse_overview_page,
            meta={"dont_cache": True},
        )

    def parse_overview_page(self, response):
        # Find other pages
        # Note that the dupefilter has to be enabled, otherwise already
        # parsed pages will be parsed again.
        for href in response.xpath(
            '//div[@id="p_main"][1]/div/a/div[@id!="p_aktuell"]/../@href'
        ):
            url = response.urljoin(href.extract())
            yield scrapy.Request(
                url, self.parse_overview_page, meta={"dont_cache": True}
            )

        # Find content
        for href in response.xpath('//a[contains(@href, "mnr")]/@href'):
            url = response.urljoin(href.extract())
            yield scrapy.Request(url, self.parse_content)

    def parse_content(self, response):
        parts = self._extract_parts(response)
        il = FeedEntryItemLoader(
            response=response, timezone="Europe/Vienna", dayfirst=True
        )
        il.add_value("path", self._library)
        il.add_value("title", " - ".join(parts[: self._find_first_meta(parts)]))
        il.add_value("link", response.url)
        il.add_xpath("updated", "//td/span/text()", re="In der Bibliothek seit: (.*)")

        _content = ["<ul>"]
        for part in parts:
            _content.append("<li>{}</li>".format(part))
        _content.append("</ul>")
        il.add_value("content_html", "".join(_content))
        return il.load_item()

    def _find_first_meta(self, parts):
        # Find the first entry after author | title | subtitle.
        for i, p in enumerate(parts):
            if p.count(",") == 2 or ":" in p:
                return i
        return len(parts)

    def _extract_parts(self, response):
        parts = [p.strip() for p in response.xpath("//td/span/text()").extract()]
        return [p for p in parts if p not in ("", ", ,")]



import json
from datetime import datetime

import scrapy

from feeds.loaders import FeedEntryItemLoader
from feeds.spiders import FeedsSpider
from feeds.utils import generate_feed_header


class ViceComSpider(FeedsSpider):
    name = "vice.com"

    def feed_headers(self):
        if not self._locales:
            return []

        for locale in self._locales:
            yield generate_feed_header(
                title="VICE {}".format(locale.title()),
                path=locale,
                link="https://www.{}".format(self.name),
                logo="https://www.{}/favicons/"
                "apple-touch-icon-60x60.png".format(self.name),
                icon="https://www.{}/favicons/"
                "apple-touch-icon-60x60.png".format(self.name),
            )

    def start_requests(self):
        self._locales = self.settings.get("FEEDS_SPIDER_VICE_COM_LOCALES")
        if not self._locales:
            self.logger.error("Please specify locale(s) for vice.com in config file!")
            return

        self._locales = self._locales.split()

        for locale in self._locales:
            yield scrapy.Request(
                "https://www.{}/api/v1/articles?locale={}".format(self.name, locale),
                meta={"locale": locale, "dont_cache": True},
            )

    def parse(self, response):
        articles = json.loads(response.text)
        remove_elems = [
            "hr + p",
            "hr",
            "iframe",
            "p i:last-of-type:contains('Facebook'):contains('Twitter')",
        ]
        for article in articles:
            il = FeedEntryItemLoader(timezone="UTC", remove_elems=remove_elems)
            il.add_value("title", article["title"])
            link = article["url"]
            if not link:
                link = "https://www.vice.com/{locale}/article/{web_id}/{slug}".format(
                    locale=response.meta["locale"],
                    web_id=article["web_id"],
                    slug=article["slug"],
                )
            il.add_value("link", link)
            if "thumbnail_url_1_1" in article:
                il.add_value(
                    "content_html",
                    '<img src="{}">'.format(article["thumbnail_url_1_1"]),
                )
            il.add_value("content_html", article["body"])
            il.add_value(
                "updated", datetime.utcfromtimestamp(article["publish_date"] / 1000)
            )
            il.add_value(
                "author_name",
                [
                    contribution["contributor"]["full_name"]
                    for contribution in article["contributions"]
                ],
            )
            il.add_value("category", article["channel"]["name"])
            for topic in article["topics"] + [article["primary_topic"]]:
                if topic and "name" in topic:
                    il.add_value("category", topic["name"].title())
            if article["nsfw"]:
                il.add_value("category", "nsfw")
            if article["nsfb"]:
                il.add_value("category", "nsfb")
            il.add_value("path", response.meta["locale"])
            yield il.load_item()



import scrapy
from scrapy.spiders import CrawlSpider, Spider, XMLFeedSpider

from feeds.cache import FeedsCache
from feeds.utils import generate_feed_header


class FeedsSpider(Spider):
    def feed_headers(self):
        yield generate_feed_header(
            title=getattr(self, "feed_title", None),
            subtitle=getattr(self, "feed_subtitle", None),
            link=getattr(self, "feed_link", None),
            path=getattr(self, "path", None),
            author_name=getattr(self, "author_name", None),
            icon=getattr(self, "feed_icon", None),
            logo=getattr(self, "feed_logo", None),
        )

    def start_requests(self):
        for url in self.start_urls:
            # Don't cache start requests otherwise we never find new content.
            yield scrapy.Request(url, dont_filter=True, meta={"dont_cache": True})

    @property
    def cache(self):
        if getattr(self, "_cache", None) is None:
            self._cache = FeedsCache(self.settings)
        return self._cache


class FeedsCrawlSpider(CrawlSpider, FeedsSpider):
    pass


class FeedsXMLFeedSpider(XMLFeedSpider, FeedsSpider):
    pass



import scrapy

from feeds.loaders import FeedEntryItemLoader
from feeds.spiders import FeedsSpider


class OpenwrtOrgSpider(FeedsSpider):
    name = "openwrt.org"
    allowed_domains = ["openwrt.org"]
    start_urls = ["https://openwrt.org/releases/start"]

    feed_title = "New OpenWrt Release Builds"
    feed_subtitle = "Newest release builds from OpenWrt."
    feed_link = f"https://{name}"
    feed_icon = f"{feed_link}/lib/tpl/openwrt/images/apple-touch-icon.png"
    feed_logo = f"{feed_link}/lib/tpl/openwrt/images/logo.png"

    def parse(self, response):
        # Page with all major releases
        xpath = '//div[@class="page group"]//a[contains(@href, "/start")]/@href'
        for href in response.xpath(xpath).extract():
            yield scrapy.Request(
                response.urljoin(href), self.parse_release, meta={"dont_cache": True}
            )

    def parse_release(self, response):
        # All minor releases per major release
        xpath = '//a[contains(@href, "notes")][text()="Release Notes"]/@href'
        for href in response.xpath(xpath):
            url = f"{self.feed_link}/{href.extract().lstrip('/')}"
            yield scrapy.Request(url, self.parse_release_notes)

    def parse_release_notes(self, response):
        il = FeedEntryItemLoader(
            response=response,
            timezone="Europe/Berlin",
            base_url=self.feed_link,
            remove_elems=[".cookielaw-banner"],
        )
        il.add_xpath("title", "//h1/text()")
        il.add_value("link", response.url)
        il.add_xpath("updated", '//div[@class="docInfo"]', re="Last modified: (.*) by")
        il.add_value("content_html", "<h1>Release Notes</h1>")
        il.add_xpath("content_html", "//h1/following-sibling::*")
        return il.load_item()



from scrapy import signals
from scrapy.downloadermiddlewares.httpcache import HttpCacheMiddleware


class FeedsHttpCacheMiddleware(HttpCacheMiddleware):
    @classmethod
    def from_crawler(cls, crawler):
        o = super().from_crawler(crawler)
        crawler.signals.connect(o.item_dropped, signal=signals.item_dropped)
        return o

    def item_dropped(self, item, response, exception, spider):
        self.storage.remove_response(response, spider)
